import re, string, os.path
import Globals

from Acquisition import aq_base
from OFS.DTMLMethod import DTMLMethod
from AccessControl import ClassSecurityInfo
from DateTime import DateTime

from Products.CMFDefault.Document import Document
from Products.CMFDefault.DublinCore import DefaultDublinCoreImpl
from Products.CMFCore.WorkflowCore import WorkflowAction
from Products.CMFCore.utils import getToolByName, _checkPermission
from Products.CMFCore.PortalContent import PortalContent
from Products.CMFCore import CMFCorePermissions
from Products.CMFCore.CMFCatalogAware import CMFCatalogAware

from Products.CMFNauTools.ManageCMFContent import ManageCMFContent
from Products.CMFCore.utils import _getViewFor


from SimpleNauItem import SimpleNauItem

factory_type_information = ( { 'id'             : 'DTMLDocument'
                             , 'meta_type'      : 'DTMLDocument'
                             , 'description'    : """\
Simple DTML-document"""
                             , 'icon'           : 'dtml_icon.gif'
                             , 'product'        : 'CMFNauTools'
                             , 'factory'        : 'addDTMLDocument'
                             , 'immediate_view' : 'metadata_edit_form'
                             , 'actions'        :
                                ( { 'id'            : 'view'
				  , 'name'          : 'View'
                                  , 'action'        : ''
                                  , 'permissions'   : (
					CMFCorePermissions.View,
                                     )
                                  }
                                , { 'id'            : 'edit'
				  , 'name'          : 'Edit'
                                  , 'action'        : 'dtml_edit_form'
                                  , 'permissions'   : (
                                      CMFCorePermissions.ModifyPortalContent, )
                                  }
                                , { 'id'            : 'metadata'
				  , 'name'          : 'Metadata'
                                  , 'action'        : 'metadata_edit_form'
                                  , 'permissions'   : (
                                      CMFCorePermissions.ModifyPortalContent, )
                                  }
                                )
                             }
                           ,
                           )

class DTMLDocument(SimpleNauItem, CMFCatalogAware, DTMLMethod, PortalContent, DefaultDublinCoreImpl, ManageCMFContent):
    """ CMF DTML document """
    meta_type = 'DTMLDocument'
    security = ClassSecurityInfo()
    effective_date = expiration_date = None
    

    security.declareProtected( CMFCorePermissions.View, 'view' )
    def view(self, REQUEST):
        """
        This method is called only from NauSite user interface (and is not called by default)
        """
        return self.dtml_edit_form( self, REQUEST )


    index_html = None  # This special value informs ZPublisher to use __call__

    def edit(self, title, data, REQUEST=None):
        if title: self.title=str(title)
        if type(data) is not type(''): data=data.read()
        self.munge(data)
        self.ZCacheable_invalidate()
        if REQUEST:
            message="Content changed."
            return self.view(self,REQUEST,potal_status_message=message)

    def upload(self,file='', REQUEST=None):
        """Replace the contents of the document with the text in file."""
        self._validateProxy(REQUEST)
        if self.wl_isLocked():
            raise ResourceLockedError, (
                'This document has been locked via WebDAV.')
        if type(file) is not type(''): file=file.read()
        self.munge(file)
        self.ZCacheable_invalidate()
        if REQUEST:
            message="Content uploaded."
            return self.dtml_edit_form(self,REQUEST,portal_status_message=message)

    def manage_afterAdd(self, item, container):

        # Automatically publish object in case it 
        # is added inside the SiteContainer instance
        workflow = getToolByName(self, 'portal_workflow')
        if hasattr(container, 'ext_link') and workflow.getInfoFor(self, 'review_state', '') != 'published':
           workflow.doActionFor( item, 'publish', comment='Published by NauSite' )

        PortalContent.manage_afterAdd(self, item, container)

Globals.InitializeClass(DTMLDocument)


default_dd_html="""<dtml-var standard_html_header>
<h2><dtml-var title_or_id></h2>
<p>
This is the <dtml-var id> DTML Document.
</p>
<dtml-var standard_html_footer>"""

def addDTMLDocument(self, id, title='', file='', REQUEST=None, submit=None):
    """Add a DTML Document object with the contents of file. If
    'file' is empty, default document text is used.
    """
    if type(file) is not type(''): file=file.read()
    if not file: file=default_dd_html
    id=str(id)
    title=str(title)
    ob=DTMLDocument(file, __name__=id)
    ob.title=title
    id=self._setObject(id, ob)
    if REQUEST is not None:
        try: u=self.DestinationURL()
        except: u=REQUEST['URL1']
        if submit==" Add and Edit ": u="%s/%s" % (u,quote(id))
        REQUEST.RESPONSE.redirect(u+'/manage_main')
    return ''
