import Globals

from Products.CMFCore.utils import UniqueObject
from OFS.SimpleItem import SimpleItem
from Acquisition import aq_inner, aq_parent

from Globals import InitializeClass, DTMLFile
from AccessControl import ClassSecurityInfo, Permissions
from AccessControl.PermissionRole import rolesForPermissionOn
from AccessControl.Role import RoleManager
from Products.CMFCore.ActionProviderBase import ActionProviderBase
from Products.CMFCore.ActionInformation import ActionInformation
from Products.CMFCore.Expression import Expression
from Products.CMFCore import CMFCorePermissions
from Products.CMFCore.utils import getToolByName, mergedLocalRoles
from Products.NuxUserGroups.UserFolderWithGroups import UserFolderWithGroups
from AccessControl.Role import classattr
from AccessControl.PermissionRole import rolesForPermissionOn
from string import join

class GroupsTool(UniqueObject, SimpleItem, ActionProviderBase):
    id = 'portal_groups'
    meta_type = 'NauSite User Groups Tool'
    _actions = [ActionInformation(id='manageGroups'
                            , title='Manage groups'
                            , description='User groups management'
                            , action=Expression(
            text='string: ${portal_url}/manage_groups_form')
                            , permissions=(CMFCorePermissions.ManagePortal,)
                            , category='user'
                            , condition=None
                            , visible=1
                             ),
                ActionInformation(id='manageUsers'
                            , title='Manage users'
                            , action=Expression(
            text='string: ${portal_url}/manage_users_form')
                            , permissions=(Permissions.manage_users,)
                            , category='user'
                            , condition=None
                            , visible=1
                             ),
                ActionInformation(id='addUser'
                            , title='Create user'
                            , action=Expression(
            text='string: ${portal_url}/join_form')
                            , permissions=(CMFCorePermissions.ManagePortal,)
                            , category='user'
                            , condition=None
                            , visible=1
                             ),
               ActionInformation(id='preferences'
                            , title='Preferences'
                            , description='Change your user preferences'
                            , action=Expression(
            text='string: ${portal_url}/personalize_form')
                            , permissions=(CMFCorePermissions.View,)
                            , category='user'
                            , condition=Expression(text='member')
                            , visible=1
                             )

] 

    security = ClassSecurityInfo()

    manage_options = ( ActionProviderBase.manage_options +
                      ({ 'label' : 'Overview', 'action' : 'manage_overview' }
                     , 
                     ) + SimpleItem.manage_options
                     )

    #
    #   'portal_groups' interface methods
    #
    security.declarePrivate('listActions')
    def listActions(self, info=None):
        """
          Return actions provided by tool.
        """
        return self._actions

    def list_groups(self, object=None):
        """
          Return a groups list
        """
        url_tool = getToolByName(self.content, 'portal_url')
	portal = url_tool.getPortalObject()
        groups = portal.acl_users.groups.keys()

        return tuple(groups)

    def group_users(self, group):
        """
          Returns the list of the group user names
        """
        url_tool = getToolByName(self.content, 'portal_url')
	portal = url_tool.getPortalObject()
        users = portal.acl_users.getGroupById(group).getUsers()

	return users

    def local_users(self):
        """
          Returns the list of all user names
        """
        url_tool = getToolByName(self.content, 'portal_url')
	portal = url_tool.getPortalObject()
        users = portal.acl_users.list_local_userids()

	return users

    def inheritedRole(self, object, group, role):
        """
          Does this role is inherited from the parent?
	"""
        if self.hasRole(object, group, role) and \
           group not in object.groups_with_local_role(role):
           return 1

    def hasRole(self, object, group, role):
	"""
          Check whether the group has 
          the given permission over the object
	"""
        roles = []
        while object.meta_type != 'NauSite':
           group_roles = object.__ac_local_group_roles__
           if group_roles:
              roles += group_roles.get(group, [])
           object = object.aq_parent

        if role in roles:
           return 'checked'

	return ''

    security.declareProtected(CMFCorePermissions.ManagePortal, 'setRoles')
    def setRoles(self, userid, roles):
	"""
          Assign roles to ther given user
	"""
        member_tool = getToolByName(self.content, 'portal_membership')
        member = member_tool.getMemberById(userid)
        member.getUser().roles = roles

        return        

    security.declareProtected(CMFCorePermissions.ManagePortal, 'manage_groups')
    def manage_groups(self, REQUEST, addGroup=None, delGroup=None):
        """
          Create or delete the user group
          Called by management screen.
	"""
	if addGroup is None and delGroup is None: 
		addGroup = REQUEST.get('addGroup', '')
		delGroup = REQUEST.get('delGroup', '')

        if addGroup:
            group=REQUEST.get('group')
            if group:
               return self._addGroup(group, REQUEST)

        if delGroup:
            groups=REQUEST.get('groups')
            return self._delGroups(groups, REQUEST)

	REQUEST['RESPONSE'].redirect(self.absolute_url() + '/manage_groups_form')

    def _addGroup(self, group, REQUEST=None):
        url_tool = getToolByName(self.content, 'portal_url')
	portal = url_tool.getPortalObject()

	portal.acl_users.userFolderAddGroup(group)

        if REQUEST is not None:
	    REQUEST['RESPONSE'].redirect(self.portal_url() + '/group_editor_form?group=' + group)


    def _delGroups(self, groups, REQUEST):
        if not groups: return 

        url_tool = getToolByName(self.content, 'portal_url')
	portal = url_tool.getPortalObject()
        portal.acl_users.userFolderDelGroups(groups)

        if REQUEST is not None:
	    REQUEST['RESPONSE'].redirect(self.portal_url() + '/manage_groups_form')


    security.declarePublic('Title')
    def Title(self, group):
        """
          Returns the group title
        """
        url_tool = getToolByName(self.content, 'portal_url')
	portal = url_tool.getPortalObject()
        title = portal.acl_users.getGroupById(group).title

	return title

    security.declareProtected(CMFCorePermissions.ChangePermissions, 'changeGroup')
    def changeGroup(self, group=None, group_users=None, title=None, REQUEST=None):
        """
           Assign the users to the given group 
           and change the group description
        """
	if group is None: 
		group = REQUEST.get('group')
		group_users = REQUEST.get('group_users', '')
		title = REQUEST.get('title', '')

	if not group_users: group_users=[]

        url_tool = getToolByName(self.content, 'portal_url')
	portal = url_tool.getPortalObject()

        portal.acl_users.setUsersOfGroup( group_users, group )

        if title is not None:
           portal.acl_users.getGroupById(group).setTitle(title)

        if REQUEST is not None:
           REQUEST['RESPONSE'].redirect(self.portal_url() + '/manage_groups_form')

    def getEmail(self, u_id):
        membership = getToolByName(self, 'portal_membership')
        member = membership.getMemberById( u_id )

	if member is not None:
           return member.email

        return None

    def getFullName(self, u_id, use_login=1):
        membership = getToolByName(self, 'portal_membership')
        member = membership.getMemberById( u_id )

	if member is not None:
                fname = member.getProperty('fname', None)
                mname = member.getProperty('mname', None)
                lname = member.getProperty('lname', None)
                fullname = join(filter(None, [fname, mname, lname]), ' ')
                if not fullname: 
                   fullname = u_id
                elif use_login:
                   fullname += ' (%s)' % u_id

		return fullname
	else:
		return u_id

    security.declarePublic('getUserInfo')
    def getUserInfo(self, u_id):
        """
          Get user properties
        """
        membership = getToolByName(self, 'portal_membership')
        member = membership.getMemberById( u_id )
        info = {}
        if hasattr(member, 'propertyIds'):
           for key in member.propertyIds():
               info[key] = member.getProperty(key)

        return info

    security.declarePublic('allowedUsers')
    def allowedUsers(self, object, roles):
        """
          Return a list of roles and users with View permission.
          Used by PortalCatalog to filter out items you're not allowed to see.
        """
        allowed = []

        membership = getToolByName(self, 'portal_membership')
        for id in self.local_users():
            member = membership.getMemberById( id )
            if member.has_role(roles, object):
               allowed.append( id )
        return allowed

InitializeClass(GroupsTool)
