import re, string, os.path
import Globals

from Acquisition import aq_base, aq_inner, aq_parent
from OFS.Folder import Folder
from AccessControl import ClassSecurityInfo
from DateTime import DateTime
from string import join

from Products.CMFDefault.Document import Document
from Products.CMFCore.WorkflowCore import WorkflowAction
from Products.CMFCore.utils import getToolByName, _checkPermission
from Products.CMFCore import CMFCorePermissions
from Products.CMFCore.PortalFolder import PortalFolder
from Products.CMFCore.CMFCatalogAware import CMFCatalogAware
from Products.CMFCore.utils import getToolByName, _getViewFor
from Products.Photo import Photo

from SimpleNauItem import SimpleNauItem
from HTMLCleanup import HTMLCleaner
from ManageCMFContent import ManageCMFContent

factory_type_information = ( { 'id'             : 'HTMLDocument'
                             , 'meta_type'      : 'HTMLDocument'
                             , 'description'    : """\
HTML-document with WYSIWYG editor"""
                             , 'icon'           : 'doc_icon.gif'
                             , 'product'        : 'CMFNauTools'
                             , 'factory'        : 'addHTMLDocument'
                             , 'immediate_view' : 'document_edit_form'
                             , 'actions'        :
                                ( { 'id'            : 'view'
				  , 'name'          : 'View'
                                  , 'action'        : 'document_view'
                                  , 'permissions'   : (
					CMFCorePermissions.View,
                                     )
                                  }
                                , { 'id'            : 'followup'
				  , 'name'          : 'Follow-up tasks'
                                  , 'action'        : 'document_follow_up_form'
                                  , 'permissions'   : (
                                      CMFCorePermissions.View, )
                                  }
                                , { 'id'            : 'edit'
				  , 'name'          : 'Edit'
                                  , 'action'        : 'document_edit_form'
                                  , 'permissions'   : (
                                      CMFCorePermissions.ModifyPortalContent, )
                                  }
                                , { 'id'            : 'metadata'
				  , 'name'          : 'Metadata'
                                  , 'action'        : 'metadata_edit_form'
                                  , 'permissions'   : (
                                      CMFCorePermissions.ModifyPortalContent, )
                                  }
                                )
                             }
                           ,
                           )

def addHTMLDocument(self, id, title='', description='', text_format='html',
                text=''):
    """ Add an HTML document """
    o = HTMLDocument(id, title, description, text_format, text)

    self._setObject(id, o)

##    # Trick to please the WYSIWYG editor
##    o2 = PortalFolder(id, '')
##    o._setObject(id,o2)

class HTMLDocument(SimpleNauItem, Document, Folder, ManageCMFContent):
    """ Subclassed folderish Document type """
    meta_type = 'HTMLDocument'
    security = ClassSecurityInfo()

    manage_options = ( Document.manage_options )

    def __call__(self):
        """\
           Invokes the default view.
        """
        if not hasattr(self, 'changes_log'):
           self.changes_log=[]

        if hasattr(self, 'REQUEST'):
           view = _getViewFor(self)
      
           if getattr(aq_base(view), 'isDocTemp', 0):
               return apply(view, (self, self.REQUEST))
           else:
               return view()

    def __init__(self, id, title='', description='', text_format='', text=''):
        Document.__init__(self, id, title, description, text_format, text)
        self.changes_log=[]

    def cookId(self, item):
        """ 
          Get id from the file path 
        """
        rgx = re.compile(r'(^.*[\\/])')
        cooked = re.sub(rgx,"",str(item.filename))
        return cooked

    def cleanup(self, text):
        # Remove absolute links to attached files, make them relative
        rgx = re.compile(r'http://[\w:]+?/%s/' % ( self.absolute_url(1)) )
#        text = re.sub(rgx,"" , text)

        # Cleanup Word HTML
        text = HTMLCleaner(text, None, 2, '', 'SCRIPT STYLE')
        return text

    def addFile(self, file, html=' ' ):
        """ 
          Add a file into the HTML document 
        """
        if not file:
            return None

        basename = self.cookId(file)

        title, suffix=os.path.splitext(basename)

        # Scan filename for illegal characters
        rgx = re.compile(r'(^_|[^a-zA-Z0-9-_~\,\.])')
        if rgx.search( basename ):
            # Place an original id to the object's title and change it to the random number
            id = str( int(DateTime().timeTime()) )

            basename = "file_%s" % id  
            if suffix:
                basename+="%s" % suffix

        # Choose the way to add the file and attach it to the document
        suffix = string.lower(suffix)
        if suffix in ['.htm', '.html'] or basename[-5:] == "_html":
            self.manage_addDTMLDocument(id=basename, title=title, file=file)
        elif suffix in ['.gif', '.jpg', '.jpeg', '.png']:
            try:
               import PIL.Image
               self.manage_addProduct['Photo'].manage_addPhoto(id=basename, title=title, file=file, engine='PIL', quality=95)
            except ImportError:
               self.manage_addImage(id=basename, title=title, file=file)
        else:
            self.manage_addFile(id=basename, title=title, file=file)
   
        html = self.pasteFile(basename, html)
        return html

    def pasteFile(self, basename, html=' ', size=None):
        """ 
           Insert the file reference into the HTML code 
           Do not touch the file itself
        """
        title, suffix=os.path.splitext(basename)

        ob = self._getOb( basename )
        if ob.title:
            title=os.path.splitext(ob.title)[0]

        if ob:
            if string.lower(suffix) in ['.htm', '.html'] or basename[-5:] == "_html":
                addon = '<a href="' + basename + '">' + title + '</a>'
            elif string.lower(suffix) in ['.gif', '.jpg', '.jpeg', '.png']:
                if size: 
                   addon = '<a href="%s"' % basename + '" target="_blank"><img src="%s?display=%s' % (basename, size) + '" border=0></a>'
                else: 
                   addon = '<img src="%s"' % basename + '" border=0> '
            else:
                addon = '<a href="%s"' % basename + '">' + title + '</a>'

            if html:
                bodyre = re.compile(r'<body.*?>', re.DOTALL|re.I)
                body = bodyre.search(html)

                if not body:
                    html = addon + html
                else: html = html[:body.end()] + addon + html[body.end():]

                return html

    def deleteFile(self, basename, html=' ' ):
        """
           Remove all links pointing to the given file and delete file
        """

        suffix=os.path.splitext(basename)[1]

        if basename:
            if string.lower(suffix) in ['.gif', '.jpg', '.jpeg', '.png']:
                delre = re.compile('<img([^<>]+?)src=([^<>]*?)%s(.*?)>' % basename, re.I|re.DOTALL)
            else:
                delre = re.compile('<a([^<>]+?)href=([^<>]*?)%s(.*?)>(.*?)</a>' % basename, re.I|re.DOTALL)

            html = delre.sub(' ', html)
            self.manage_delObjects(basename)

            return html

    def manage_afterClone(self, item):
        """
          Notify and reindex
        """
        if aq_base(item) is aq_base(self):
            wf = getToolByName(self, 'portal_workflow', None)
            if wf is not None:
                wf.notifyCreated(self)

        # Reset local roles and assign a new owner
        old_owner = self.users_with_local_role('Owner')
        self.manage_delLocalRoles(old_owner)
        self.manage_setLocalRoles(item.Creator(), ['Owner'])

        item.reindexObject()
        
        return

    def objectIds(self, spec=None):
        if spec is not None:
            if type(spec)==type('s'):
                spec=[spec]
            set=[]

            for ob in self._objects:
                if ob['meta_type'] in spec:
                    set.append(ob['id'])
            return set

        set = filter(lambda x: x['meta_type'] not in ['Portal Folder', 'Folder'], self._objects)

        return map(lambda i: i['id'], set)

    def objectValues(self, spec=None):
        return map(self._getOb, self.objectIds(spec))

    def objectItems(self, spec=None):
        r=[]
        a=r.append
        g=self._getOb
        for id in self.objectIds(spec): 
            if id != self.id: a((id, g(id)))
        return r

    #
    # Document review support
    def requestConfirmation(self, requested_users, confirm_by_turn, comment=''):
          """\
             Request the user's confirmation     
          """
          if confirm_by_turn:
                requested_users=[requested_users[0], ]

          mto = self.getEmail( requested_users, 0 )
          mfrom = self.portal_properties.email_from_address
          msubject = ' '
          mailtext = 'To: ' + join( mto, ', ') + '\n' + \
                     ' ' + self.portal_membership.getAuthenticatedMember().getUserName() + ' ' + \
                     '    "' + self.title_or_id() + '"\n' + \
                     'URL: ' + self.absolute_url() + '\n\n' + \
                     ': ' + comment + '\n' + \
                     '\n'
          try:
            self.MailHost.send(mailtext, mto, mfrom, msubject)
          except:
            pass

          for member_id in requested_users: 
                 if 'Owner' not in self.get_local_roles_for_userid(member_id):
                          self.manage_setLocalRoles(member_id, ['Reviewer'])


    security.declareProtected( CMFCorePermissions.ManageProperties, 'cancelConfirmationRequest')
    def cancelConfirmationRequest(self, comment=''):
          """\
             Cancel the user's confirmation request
          """
          requested_users = list( self.getProperty('requested_users') )
          confirmed_users = list( self.getProperty('confirmed_users', '') )

          mto = self.getEmail( requested_users + confirmed_users, 0 )
          mfrom = self.portal_properties.email_from_address
          msubject = ' '
          mailtext = 'To: ' + join(mto, ', ') + '\n' + \
                     ' ' + self.portal_membership.getAuthenticatedMember().getUserName() + ' ' + \
                     '      "' + self.title_or_id() + '"\n' + \
                     'URL: ' + self.absolute_url() + '\n\n' + \
                     ': ' + comment + '\n' + \
                     '\n'
          try:
            self.MailHost.send(mailtext, mto, mfrom, msubject)
          except:
            pass

          member_ids = self.users_with_local_role('Reviewer')
          self.manage_delLocalRoles(member_ids)

          try:
            self.manage_delProperties(['confirmed_users', 'requested_users', ])
          except:
            pass


    security.declareProtected( CMFCorePermissions.ReviewPortalContent, 'confirm')
    def confirm(self, comment='', REQUEST=None):
          """\
             User confirms the document
          """
          membership = getToolByName(self, 'portal_membership')
          uname = membership.getAuthenticatedMember().getUserName()

          message='Error+while+review.'

          if self.hasProperty('requested_users') and uname in self.getProperty('requested_users'):
               requested_users = list( self.getProperty('requested_users') )
               confirmed_users = filter(None, list(self.getProperty('confirmed_users', [])) )
               
               mto = self.getEmail( ( self.Creator(),  ), 0 )
               mfrom = self.portal_properties.email_from_address
               msubject = ' '
               mailtext = 'To: ' + join( mto, ',') + '\n' + \
                          ' "' + self.title_or_id() + '"\n' + \
                          '   ' + self.portal_membership.getAuthenticatedMember().getUserName() + '\n' + \
                          'URL: ' + self.absolute_url() + '\n\n' + \
                          ': ' + comment + '\n' + \
                          '\n'

               if uname not in confirmed_users: confirmed_users.append( uname )
               requested_users.remove( uname )

               try:
                  self.manage_addProperty('confirmed_users', '', 'lines')
               except:
                  pass

               self.manage_changeProperties(confirmed_users=confirmed_users)
               self.manage_changeProperties(requested_users=requested_users)

               message = 'You confirmation processed'

               confirm_by_turn=self.getProperty('confirm_by_turn', '')

               if not requested_users:
                   message = 'Document confirmed'

                   mailtext = 'To: ' + join( mto, ',') + \
                              ' "' + self.title_or_id() + '"   .\n' + \
                              'URL: ' + self.absolute_url() + '\n\n' + \
                              ': ' + comment + '\n' + \
                              '\n'

                   self.portal_workflow.doActionFor( self, 'fix_reviewed', comment='  : ' + join(confirmed_users, ', ') )
##                   member_ids = self.users_with_local_role('Reviewer')
##                   self.manage_delLocalRoles(member_ids)
               elif confirm_by_turn:
                      self.requestConfirmation(requested_users, confirm_by_turn)


               try:
                 self.MailHost.send(mailtext, mto, mfrom, msubject)
               except:
                 message+=" (no mail was sent)"

          if REQUEST is not None:
              REQUEST[ 'RESPONSE' ].redirect( self.absolute_url()
                            + '?portal_status_message=' + message
                            )

    security.declareProtected( CMFCorePermissions.ReviewPortalContent, 'refuse')
    def refuse(self, comment='', REQUEST=None):
          """\
             User refuses confirmation
          """
          membership = getToolByName(self, 'portal_membership')
          uname = membership.getAuthenticatedMember().getUserName()
          message='Error+while+review.'

          if self.hasProperty('requested_users') and uname in self.getProperty('requested_users'):
               message = 'Confirmation+refused'

               requested_users = list( self.getProperty('requested_users') )
               confirmed_users = list( self.getProperty('confirmed_users', '') )
               refused_users = list( self.getProperty('refused_users', '') )

               mto = self.getEmail( [self.Creator(), ] + requested_users + confirmed_users, 0 )

               mfrom = self.portal_properties.email_from_address
               msubject = '  '
               mailtext = 'To: ' + join(mto, ',') + '\n' + \
                          ' ' + uname + \
                          '     "' + self.title_or_id() + '"\n' + \
                          'URL: ' + self.absolute_url() + '\n\n' + \
                          ': ' + comment + '\n' + \
                          '\n'

               if uname not in refused_users: refused_users.append( uname )
               requested_users.remove( uname )

               try:
                  self.manage_addProperty('refused_users', '', 'lines')
               except:
                  pass

               self.manage_changeProperties(refused_users=refused_users)
               self.manage_changeProperties(requested_users=requested_users)

               self.portal_workflow.doActionFor( self, 'refuse', comment='Confirmation refused by ' + uname )

               try:
                 self.MailHost.send(mailtext, mto, mfrom, msubject)
               except:
                 message+=" (no mail was sent)"

          if REQUEST is not None:
              REQUEST[ 'RESPONSE' ].redirect( self.absolute_url()
                            + '?portal_status_message=' + message
                            )


    #
    # Document rating support
    security.declareProtected(CMFCorePermissions.View, 'rate')
    def rate(self, grade, REQUEST=None):
        """
          Rate the document
        """
        if grade:
           membership = getToolByName(self, 'portal_membership')
           member = membership.getAuthenticatedMember()
           uname = member.getUserName()
           try:
             self.manage_addProperty('rate_sum', 0, 'int')
             self.manage_addProperty('rate_users', '', 'tokens')
           except:
             pass

##           try:
##             member.manage_addProperty('rate_sum', 0, 'int')
##             member.manage_addProperty('rate_users', '', 'tokens')
##           except:
##             pass
   
           rate_sum = self.getProperty('rate_sum', 0)
           rate_users = self.getProperty('rate_users', '')                     
   
           if uname not in rate_users:
              self.manage_changeProperties( rate_sum = rate_sum + int(grade) )
              self.manage_changeProperties( rate_user = rate_users.append(uname) )
   
           if REQUEST is not None:
               REQUEST[ 'RESPONSE' ].redirect( self.absolute_url() )        

    security.declareProtected(CMFCorePermissions.View, 'rated')
    def rated(self, REQUEST=None):
        """
          Checks whether the user rated this document
        """
        membership = getToolByName(self, 'portal_membership')
        uname = membership.getAuthenticatedMember().getUserName()

        rate_sum = self.getProperty('rate_sum', 0)
        rate_users = self.getProperty('rate_users', '')
   
        if rate_users and uname in rate_users:
           return 1

        return 

    security.declareProtected(CMFCorePermissions.View, 'get_rating')
    def get_rating(self):
        """
          Get the document rating
        """
        rate_sum = self.getProperty('rate_sum', 0)
        rate_users = self.getProperty('rate_users', '')

        if len(rate_users) > 0:
           return rate_sum / len(rate_users)

        return 0

## Not wanted anymore    
##
##    security.declarePublic('reindexObject')
##    def reindexObject(self):
##        """
##          Update local and external catalogues for the object 
##        """
##        catalog = getToolByName(self, 'portal_catalog', None)
##        if catalog is not None:
##            catalog.reindexObject(self)
##    
##        workflow_tool = getToolByName(self, 'portal_workflow')
##        review_state = workflow_tool.getInfoFor(self, 'review_state')
##
##
##        # reindex published items directly in the publishers
##        if review_state == 'published':
##           publishers = filter( None, self.portal_url.publishers)
##    
##           for path in publishers:
##              publisher = self.unrestrictedTraverse( path )
##              catalog = getToolByName(publisher, 'portal_catalog', None)
##              if catalog is not None:
##                 internal_url = join(self.getPhysicalPath(), '/')
##                 external_url = internal_url.replace( publisher.internal, path )
##                 publisher.portal_catalog.catalog_object(self, external_url )


    security.declarePublic( 'getTasksFor' )
    def getTasksFor(self, content):
        """
            Return the report for content, creating it if need be.
        """
        followup = getattr( content, 'followup', None )
        if not followup:
             from TaskItem import TasksContainer

             content.followup = TasksContainer()
             followup = content.followup
        
        return followup

Globals.InitializeClass(HTMLDocument)

def install(target):
     """\
        Registering HTML Document class
     """
     Installer = ManageCMFContent()

     Installer.deploy_class( target=target
                      , type_name='HTMLDocument'
                      , fti=factory_type_information
                      )
