import Globals

from AccessControl import ClassSecurityInfo
from Globals import InitializeClass, DTMLFile, HTML
from Acquisition import aq_parent, aq_base, aq_self
from Products.MailHost.MailHost import MailHost
from urllib import quote
from urllib2 import urlopen
from string import join
import re

from Products.CMFCore.utils import _getViewFor
from Products.CMFCore.CMFCorePermissions import View, ManageProperties, ListFolderContents
from Products.CMFCore.CMFCorePermissions import AddPortalFolders, AddPortalContent, ChangePermissions
from Products.CMFCore.PortalFolder import PortalFolder
from Products.CMFCore.CMFCatalogAware import CMFCatalogAware
from Products.CMFCore import CMFCorePermissions
from Products.CMFCore.utils import getToolByName
from Products.CMFCore.Expression import Expression

from Products.CMFDefault.DublinCore import DefaultDublinCoreImpl

from ManageCMFContent import ManageCMFContent
from SimpleNauItem import SimpleNauItem

factory_type_information = ( { 'id'             : 'Heading'
                             , 'meta_type'      : 'Heading'
                             , 'description'    : """\
Heading is a kind of folder that is used to store documents and other CMF objects."""
                             , 'icon'           : 'folder_icon.gif'
                             , 'product'        : 'CMFNauTools'
                             , 'factory'        : 'manage_addHeading'
                             , 'filter_content_types' : 0
                             , 'immediate_view' : 'folder_edit_form'
                             , 'actions'        :
                                ( { 'id'            : 'view'
                                  , 'name'          : 'View'
                                  , 'action'        : 'folder_contents'
                                  , 'permissions'   : (View,
                                     )
                                  , 'category'      : 'folder'
                                  }
                                , { 'id'            : 'edit'
                                  , 'name'          : 'Properties'
                                  , 'action'        : 'folder_edit_form'
                                  , 'permissions'   : (ChangePermissions,)
                                  , 'category'      : 'folder'
                                  }
                                , { 'id'            : 'filter'
                                  , 'name'          : 'Content filter'
                                  , 'action'        : 'folder_contents?open_filter_form=1'
                                  , 'permissions'   : (View,)
                                  , 'category'      : 'folder'
                                  }
                                , { 'id'            : 'accessgroups'
                                  , 'name'          : 'Access control'
                                  , 'action'        : 'manage_access_form'
                                  , 'permissions'   : (ChangePermissions,)
                                  , 'category'      : 'folder'
                                  }
                                )
                             }
                           ,
                           )

ManagedRoles = ['Editor', 'Reader', 'Writer', 'Author', ]

class Heading( SimpleNauItem, CMFCatalogAware, PortalFolder, ManageCMFContent, DefaultDublinCoreImpl ):
    """ Heading class """
    meta_type = 'Heading'
    portal_type = 'Heading'

    ManagedRoles = ManagedRoles

    security = ClassSecurityInfo()

    def __call__(self):
        """\
           Invokes the default view.
        """
        if hasattr(self, 'REQUEST'):
           view = _getViewFor(self)
      
           if getattr(aq_base(view), 'isDocTemp', 0):
               return apply(view, (self, self.REQUEST))
           else:
               return view()

    security.declareProtected( View, 'view' )
    view = __call__

    index_html = None

    def __init__(self, id, title=''):
        self.id = id
        self.title = title
	self.subscribed_users = []
	self.pending_subscriptions = {}
	
    def hasMainPage(self):
        """
           Check whether this folder is in main page mode
        """
        mode = getattr(self, 'display_mode', 0)

        return mode == 1

    def setDisplayMode(self, mode=0):
        """Set mark on the topic, that some of it's documents
           must be used as index page, when publishing to external site.

           mode values:
              0 - documents list
              1 - main page
        """
        self.display_mode = mode

        return

    def getMainPage(self):
        """Returns index document for topic
           or last created, if there is no marked document"""
        isPublished = lambda doc: doc.isPublished()
        published_docs = filter(isPublished, self.objectValues())

        array_with_times = [(doc.Date(), doc)
                            for doc in published_docs]
        array_with_times.sort()
        just_docs = [pair[1] for pair in array_with_times]
        result = None
        for doc in just_docs:
            result = doc
            if callable(getattr(doc, 'isIndexDocument', None)) \
               and doc.isIndexDocument(): break 

        return result

    security.declareProtected( AddPortalFolders, 'manage_addHeading')
    def manage_addHeading(self, id, title='', REQUEST=None):
        """\
           Add a new Heading object with id *id*.
        """
        ob=Heading(id, title)
        self._setObject(id, ob)
        self.reindexObject()

        if REQUEST is not None:
            return self.folder_contents( # XXX: ick!
                self, REQUEST, portal_status_message="Topic added")

    def manage_afterClone(self, item):
        """
          Notify and reindex after the copy/paste sequence
        """
        if aq_base(item) is aq_base(self):
            wf = getToolByName(self, 'portal_workflow', None)
            if wf is not None:
                wf.notifyCreated(self)

        if hasattr(item, 'reindexObject'):
            item.reindexObject()

        if hasattr(item, 'reindexSubObjects'):
            item.reindexSubObjects()

        return

    security.declareProtected( CMFCorePermissions.ChangePermissions, 'manage_permissions')
    def manage_permissions(self, REQUEST):
	""" 
	   Change permissions of the given object
	"""
        url_tool = getToolByName(self.content, 'portal_url')
	portal = url_tool.getPortalObject()
	
        groups_range = range(len(portal.acl_users.groups.keys()))
        roles = filter( lambda x: x in ManagedRoles, portal.valid_roles())

        have=REQUEST.has_key
        local_roles = {}

        # Read the desirable configuration from REQUEST
        for ir in groups_range:
          for role in roles:
             if have("%s_%d" % (role,ir)):
                group = REQUEST.get("%s_%d" % (role,ir))
                group_roles = local_roles.get(group, [])
                group_roles.append(role)
                local_roles[group] = group_roles

        # Reset local roles settings
        groups = portal.acl_users.groups.keys()
        self.manage_delLocalGroupRoles(groups)

        # Assign local roles to the groups
        for group in local_roles.keys():
            self.manage_setLocalGroupRoles(group, local_roles[group]) 

        self.reindexSubObjects()
	
	if REQUEST is not None:
		REQUEST['RESPONSE'].redirect(self.absolute_url() + '/manage_access_form')
	return

    security.declareProtected(CMFCorePermissions.ChangePermissions, 'reindexSubObjects')
    def reindexSubObjects(self):
        """
	   Reindex objects inside the given folder 
	   (have to update 'allowedRolesAndUsers')
	"""
        for ob in self.objectValues():
	   if hasattr( aq_self(ob), 'reindexObject'):
               ob.reindexObject()
	   if hasattr( aq_self(ob), 'reindexSubObjects'):
               ob.reindexSubObjects()

        return

    security.declareProtected(CMFCorePermissions.ChangePermissions, 'setLocalRoles')
    def setLocalRoles(self, userid, roles, REQUEST=None):
        """
           Wrapper for the built-in manage_setLocalRoles
        """
        if userid:
           user_roles = self.getLocalRoles( userid )
           
           # don't assign local roles twice
           if user_roles:
              roles = filter( lambda x, user_roles=user_roles[0]: x not in user_roles, roles )
           self.manage_setLocalRoles(userid, roles)
           
           # Update allowedRolesAndUsers ZCatalog field
           self.reindexSubObjects()

        if REQUEST is not None:
            REQUEST[ 'RESPONSE' ].redirect(self.absolute_url() + '/manage_access_form') 

    security.declareProtected(CMFCorePermissions.ChangePermissions, 'delLocalRoles')
    def delLocalRoles(self, userids, REQUEST=None):
        """
           Wrapper for the built-in manage_delLocalRoles
        """
        self.manage_delLocalRoles(userids)

        # Update allowedRolesAndUsers ZCatalog field
	self.reindexSubObjects()

        if REQUEST is not None:
            REQUEST[ 'RESPONSE' ].redirect(self.absolute_url() + '/manage_access_form') 

    security.declareProtected(CMFCorePermissions.ChangePermissions, 'getLocalRoles')
    def getLocalRoles(self, userid=None):
        """
           Return all local roles assigned over this object
        """
        object = self
        info=[]
        # walk through the whole CMF Site and gather all
        # acquired roles that are applied to the object
        while object.meta_type != 'NauSite':        
            dict=object.__ac_local_roles__ or {}
            keys=dict.keys()
            keys.sort()
            # user is a dict key, value
            for key in keys:
                # be aware about ManagedRoles only
                value=list(filter(lambda x: x in ManagedRoles, dict[key]))
                if value:
                   if userid == key:
                      info.append(value)
                   elif userid is None:
                      if object is not self: 
                          # mark inherited roles
                          value.insert(0, '__inherited')
                      info.append((key, value))
            object = object.aq_parent
        return tuple(info)

    def user_roles(self):
        """\
           Get the users' role title in the given folder
        """
        membership = getToolByName(self, 'portal_membership')
        roles = membership.getAuthenticatedMember().getRolesInContext(self)
        roles = filter( lambda x: x in ManagedRoles, roles )

        return roles
	  
    def getBadLinks(self):
	"""
	    this function return list of documents and bad links,
	    included in given documents in format
	    [[document, links_count, [bad_link1, ...]], ...]
	"""
	import sys
	expr = '<[\s]*a[^<^>]+href[\s]*=[\s]*"([^"]+)"[^<^>]*>[^<]*<[\s]*/a[\s]*>'
	url_expr = re.compile(expr, re.IGNORECASE)
	res = []
	
	for element in self.objectValues():
	    if element.meta_type in ['HTMLDocument']:
	        badLinks = []
		link = url_expr.search( element.EditableBody())
		while link:
		    url = link.groups()[0]
		    try:
			urlopen(url)
		    except:
			err=sys.exc_info()
			if not hasattr(err[1], 'code') or err[1].code!=401:
			    badLinks.append(link.groups()[0])
		    link = url_expr.search(element.EditableBody(), link.end())
		if len(badLinks)>0:
	    	    res.append([element, len(badLinks), badLinks])
	    elif element.meta_type == 'Heading':
	        res = res + element.getBadLinks()
	return res


    #
    # Subscription interface methods
    #
    def subscribe(self, email, REQUEST=None):
	"""
          Subscribe user to news on current heading (request confirmation)
        """

	ob = self
        # Walk to the top of the site and ensure that user
        # has not subscribed to the current folder yet
	while ob and ob.meta_type not in ['CMF Site', 'NauSite']:
            base = ob.aq_base
	    if base.meta_type == 'Heading':
		if hasattr(base, 'subscribed_users') and type(base.subscribed_users) == type([]):
		    if base.subscribed_users and email in base.subscribed_users:
			return 'You already recieve news from this or parent folder'
		else:
                    # Patch old heading objects
		    ob.subscribed_users = []
		    ob._p_changed = 1
            ob = ob.aq_parent
	    
	if email:
            if not hasattr(self.aq_base, 'pending_subscriptions'):
                self.pending_subscriptions = {}

            secret_code = str(int(self.ZopeTime()))
	    self.pending_subscriptions[email] = secret_code
	    self._p_changed = 1
            # Generate an url that must be called to confirm subscription
            subscription_url = self._subscription_url({      'action': 'confirm'
                                                      ,       'email': email
                                                      , 'secret_code': secret_code
                                                      }, REQUEST)

            # Send a mail to the user
	    lang = self.msg.get_selected_language()
            mail_text = self.subscription_on(self, REQUEST, 
                                             to = email, 
                                             heading_title = self.title,
                                             subscription_url = subscription_url,
                                             lang = lang
                                            )
            host = self.MailHost
            host.send(mail_text)

	    return 'Your subscription processed. Check your e-mail and confirm subscription to begin recieving news in this topic.'
	else: 
	    return 'Invalid e-mail'

    def confirm_subscription(self, email, secret_code, REQUEST):
	##accept user subscription for news on this heading
        pending = self.pending_subscriptions

        # Ensure that given secret code matches previously set 
        # value from pending_subscriptions
        if pending.has_key(email) and pending[email] == str(secret_code):
            del pending[email]
            self.subscribed_users.append(email)
            self._p_changed = 1

            # Generate an url that must be called to cancel subscription
            subscription_url = self._subscription_url({      'action': 'unsubscribe'
                                                      ,       'email': email
                                                      }, REQUEST)


            # Send a mail to the user
	    lang = self.msg.get_selected_language()
            mail_text = self.subscription_confirm(self, REQUEST,
                                                  to = email,
                                                  heading_title = self.title,
                                                  subscription_url = subscription_url,
                                                  lang = lang
                                                 )
            host = self.MailHost
            host.send(mail_text)

            return 'Now you will recieve update notifications for this topic.'

        return 'You have not signed up for the updates mailing. Try to subscribe again.'

    def unsubscribe(self, email, REQUEST=None):
	##unsubscribe user from news in heading
        su = self.subscribed_users
        if email in su:
            su.remove(email)
            self._p_changed = 1

            # Send a mail to the user
	    lang = self.msg.get_selected_language()
            mail_text = self.subscription_off(self, REQUEST,
                                              to = email,
                                              heading_title = self.title,
                                              lang = lang
                                             )
            host = self.MailHost
            host.send(mail_text)

            return 'You were successfuly unsubscribed'
        
        return 'You are not subscribed to this topic'



    ##send announcment about new publication to subscribed users 
    def announce_publication(self, ob, banned_emails=[], REQUEST=None):
        if self.getSite() is None:
           return

        document_title_or_id = ob.title_or_id()
        document_url = ob.external_url()
        subscribed_users = ob.subscribed_users
        summary = ob.Description()

	for email in subscribed_users:
           if email not in banned_emails:
            # Generate an url that must be called to cancel subscription
            subscription_url = self._subscription_url({      'action': 'unsubscribe'
                                                      ,       'email': email
                                                      }, REQUEST)

	    lang = self.msg.get_selected_language()
            mail_text = self.subscription_announce(self, REQUEST,
                                                   to = email,
                                                   subscription_url = subscription_url,
                                                   heading_title = self.title,
                                                   document_title_or_id = document_title_or_id,
                                                   document_url = document_url,
                                                   summary = summary,
                                                   lang = lang
                                                  )
            host = self.MailHost
            host.send(mail_text)
            banned_emails.append(email)

	if self.aq_parent.meta_type == 'Heading':
	    self.aq_parent.announce_publication(ob, banned_emails, REQUEST)

    ##generate url for subscription mail	
    def _subscription_url(self, args={}, REQUEST=None):
        server_url = REQUEST['SERVER_URL']
        url =  server_url + self.external_url() + '/subscription?' + join(map(lambda x, args=args: '%s=%s' % (x, quote(args[x])), args.keys()), '&')
		
	return url 

Globals.InitializeClass(Heading)

addHeading = Heading.manage_addHeading

def install(target):
     """\
        Register the Heading class
     """
     Installer = ManageCMFContent()

     Installer.deploy_class( target=target
                      , type_name='Heading'
                      , fti=factory_type_information
                      )

