import Globals
from Globals import InitializeClass, DTMLFile
from AccessControl import ClassSecurityInfo
from Products.CMFCore import CMFCorePermissions
from Products.CMFCore.ActionInformation import ActionInformation
from Products.CMFCore.Expression import Expression
from Products.CMFDefault.MetadataTool import MetadataTool
from Products.CMFCore.PortalFolder import ContentFilter
from Products.CMFCore.utils import getToolByName

class NauMetadataTool( MetadataTool ):

    id              = 'portal_metadata'
    meta_type       = 'NauSite Metadata Tool'
    _actions = [ActionInformation(id='manageSubjects'
                            , title='Document types'
                            , description='Manage valid document types'
                            , action=Expression(
            text='string: ${portal_url}/metadata_subjects_form')
                            , permissions=(CMFCorePermissions.ManagePortal,)
                            , category='global'
                            , condition=None
                            , visible=1
                             ),
                ActionInformation(id='search'
                            , title='Search'
                            , action=Expression(
            text='string: ${folder_url}/search_form')
                            , permissions=(CMFCorePermissions.View,)
                            , category='global'
                            , condition=None
                            , visible=1
                             )] 

    security = ClassSecurityInfo()

    security.declarePrivate('listActions')
    def listActions(self, info=None):
        """
          Return actions provided by tool.
        """
        return self._actions

    security.declareProtected( CMFCorePermissions.View
                             , 'getSpecVocabulary' )
    def getSpecVocabulary(self, element, typ):
        spec = self.getElementSpec(element)
        policy = spec.getPolicy( typ )
        vocab = policy.allowedVocabulary()
        return vocab

    security.declareProtected( CMFCorePermissions.ManagePortal
                             , 'setSpecVocabulary' )
    def setSpecVocabulary(self, element, typ, allowed_vocabulary=[], REQUEST=None):
        catalog = getToolByName(self, 'portal_catalog')

        allowed_vocabulary=filter(None, allowed_vocabulary)
        old_vocab = self.getSpecVocabulary(element, typ)

        # Get the list of subjects that are currently in use and 
        # add them to allowed_vocabulary again
        for subject in old_vocab:
             subjects = len(catalog.searchResults( Subject={ 'query':subject, } ) )
             if subjects > 0:
                   if subject not in allowed_vocabulary: allowed_vocabulary.append(subject)

        spec = self.getElementSpec(element)
        policy = spec.getPolicy( typ )

        policy.allowed_vocabulary = tuple( allowed_vocabulary )
        if REQUEST is not None:
            REQUEST[ 'RESPONSE' ].redirect( self.absolute_url()
               + '/metadata_subjects_form'
               + '?element=' + element
               + '&typ=' + typ
               )

InitializeClass( NauMetadataTool )
