# $Id: NauPublishTool.py,v 1.39 2002/09/25 14:42:00 ikuleshov Exp $

import Globals
from Globals import InitializeClass, DTMLFile, HTML
from AccessControl import ClassSecurityInfo, getSecurityManager
from Products.CMFCore import CMFCorePermissions
from Products.CMFCore.ActionInformation import ActionInformation
from Products.CMFCore.ActionProviderBase import ActionProviderBase
from Products.CMFCore.Expression import Expression
from Products.CMFCore.utils import UniqueObject, getToolByName
from Products.CMFCore import CMFCorePermissions
from Products.CMFCore.PortalContent import PortalContent
from Products.CMFCore.utils import _checkPermission

from string import join, split, replace

from Products.CMFCore.utils import _getViewFor
from Acquisition import aq_base, aq_inner, aq_parent
 
from HTMLDocument import HTMLDocument
from ManageCMFContent import ManageCMFContent

from OFS.SimpleItem import SimpleItem
from os import listdir

import HTMLDocument, Heading

factory_type_information = ( { 'id'             : 'Publisher Entry Point'
                             , 'meta_type'      : 'Publisher Entry Point'
                             , 'description'    : """\
Publisher Entry Point fetches published documents for the external site"""
                             , 'icon'           : 'publisher.gif'
                             , 'product'        : 'CMFNauTools'
                             , 'factory'        : 'manage_addPublisherEP'
                             , 'filter_content_types' : 0
                             , 'immediate_view' : 'view'
                             }
                           ,
                           )

class NauPublishTool(UniqueObject, SimpleItem, ActionProviderBase):
    id              = 'portal_publisher'
    meta_type       = 'NauSite Publisher Tool'

    zeohost         = ''
    zeoport         = 0

    default_repository = ''
    soft_links      = []

    _actions = [ActionInformation(id='constructor'
                            , title='Site constructor'
                            , action=Expression(
            text='string: ${folder_url}/site_constructor_form')
                            , category='global'
                            , permissions=(CMFCorePermissions.ManagePortal,)
                            , visible=1
                            )] 

    security = ClassSecurityInfo()

    security.declareProtected( CMFCorePermissions.ManagePortal
                             , 'manage_overview' )
    manage_overview = DTMLFile( 'dtml/explainPublishTool', globals())

    security.declareProtected( CMFCorePermissions.ManagePortal
                             , 'manage_external' )
    manage_external = DTMLFile( 'dtml/setupExternalSites', globals())

    manage=manage_main=DTMLFile('dtml/setupZEOSyncer', globals())
    manage_main._setName('manage_main')

    manage_options = (({ 'label' : 'ZEOSyncer', 'action' : 'manage_main' },
                       { 'label' : 'External sites', 'action' : 'manage_external' },                       
                       { 'label' : 'Overview', 'action' : 'manage_overview' }
                      
                     , 
                     )+ ActionProviderBase.manage_options 
                      + SimpleItem.manage_options
                     )

    def __init__( self ):
        self.default_repository = 'PORTAL_ROOT/external'
        self.soft_links = ['portal_skins/custom', 'portal_skins/images']


    security.declarePrivate('listActions')
    def listActions(self, info=None):
        """
          Return actions provided by tool.
        """
        _actions = self._actions
#        points =  self.portal_catalog.searchResults( meta_type='Publisher Entry Point' )
#        for point in points:
#            _actions += point.getObject().listActions(info) 

        return _actions

    security.declareProtected( CMFCorePermissions.ManagePortal, 'listDirectories' )
    def listDirectories( self, site_path=None):
        """
          List subdirectories of the given object
        """
        if site_path is None: site_path = self.default_site
        site = self.restrictedTraverse( site_path, None )

        return

    security.declareProtected( CMFCorePermissions.ManagePortal, 'editZEOProperties' )
    def editZEOProperties( self, zeohost='localhost', zeoport=0, REQUEST=None):
        """
          Form handler for the publisher settings
        """
        msg = "Tool updated."
        self.zeohost = zeohost
        self.zeoport = zeoport

        if REQUEST is not None:
            return self.manage_main( self, REQUEST, manage_tabs_message = msg, management_view='ZEOSyncer' )

    security.declareProtected( CMFCorePermissions.ManagePortal, 'get_repository' )
    def get_repository( self ):
        """
            Return the path to the default sites repository
        """
        default_repository = self.default_repository.replace('PORTAL_ROOT', '/' + self.portal_url(1))

        return default_repository

    ## Is obsolete now
    security.declareProtected( CMFCorePermissions.ManagePortal, 'construct' )
    def construct( self, id, title='', soft_links=[], storage_name='storage', target=None, REQUEST=None):
        """
            Create and customize a CMF Site
        """
        repository_url = REQUEST['repository']
        repository = self.unrestrictedTraverse( repository_url, None )

        if repository is not None:
            target.manage_addProduct['CMFNauTools'].manage_addSiteContainer(id, title)

        return

    security.declareProtected( CMFCorePermissions.ManagePortal, 'editExternalProperties' )
    def editExternalProperties( self, soft_links='', default_repository='', REQUEST=None):
        """
            Form handler for the publisher settings
        """
        msg = "Tool updated."

        self.soft_links = soft_links
        rendered_repository = default_repository.replace('PORTAL_ROOT', '/' + self.portal_url(1))
        
        try:
           folder = self.restrictedTraverse( rendered_repository )
           if not getattr(folder, 'isPrincipiaFolderish', 0):
               raise ValueError, "%s is not a valid CMF folder" % rendered_repository

           self.default_repository = default_repository
        except:
           msg = "%s is not a valid CMF folder" % default_repository


        if REQUEST is not None:
            return self.manage_external( self, REQUEST, manage_tabs_message = msg, management_view='External sites' )

InitializeClass(NauPublishTool)

class NauPublisher(PortalContent, ManageCMFContent):
    meta_type = 'Publisher Entry Point'
    internal = ''
    isPrincipiaFolderish = 1

    manage_options = (({ 'label' : 'Publisher settings', 'action' : 'manage_main' }
                     , 
                     ) + SimpleItem.manage_options
                     )
    security = ClassSecurityInfo()

    security.declareProtected( CMFCorePermissions.ManagePortal
                             , 'manage_main' )
    manage=manage_main=DTMLFile('dtml/setupPublisherEP', globals())
    manage_main._setName('manage_main')
    index_html = None

    def __init__( self, id, title = '', internal=''):
        self.id = id
        self.title = title
        self.internal = internal

    def __before_publishing_traverse__(self, self2,  REQUEST=None):
        path = REQUEST['TraversalRequestNameStack']
        subpath = path[:]

        path[:] = []
    	
        subpath.reverse()
        REQUEST.set('traverse_subpath', subpath)
        
    def __call__(self, REQUEST):
        """\
           Default view
        """
        traverse_subpath = REQUEST['traverse_subpath']
        subpath = join(traverse_subpath, '/')
        if not subpath:
           home_page = self.getSite().getHomePage()
           if home_page:
              return self.fetchObject(home_page.absolute_url(1), REQUEST)
           
           # Default site page
           return self.listPublishedForm(self, REQUEST)

        path = self.getSiteUrl() + '/' + subpath

        try:
           result = self.fetchObject( path, REQUEST )
        except AttributeError:
           meth = traverse_subpath and traverse_subpath[-1]
           REQUEST['traverse_subpath'] = traverse_subpath[:-1]
           result = apply(getattr(self, meth), (REQUEST, ))
        except:
           result = self.site_error_message(self, REQUEST, path=path)

        return result

    ## Is obsolete. User SimpleNauItem::external_url instead
    security.declarePublic('external_url')
    def external_url(self, item=None, internal_url=None):
        """
          Apply new URL rules
        """
        if internal_url is None:
            internal_url = item.absolute_url()

	root = self.getPhysicalRoot().absolute_url()
        external_url = internal_url.replace( root + self.getSiteUrl(), root + '/' + self.absolute_url(1) )

        return external_url

    security.declarePrivate('listPublications')
    def listPublications(self, path, exact = None ):
        path_index = ((exact is not None) and 'parent_url') or 'path'

        search_args = { 'review_state': 'published'
                      , 'meta_type': ['HTMLDocument', 'DTMLDocument']
                      , path_index: path
                      }

        catalog = getToolByName(self.content, 'portal_catalog')
        results = apply(catalog.searchResults, (), search_args )

        return results
        

    security.declarePublic('publishedItems')
    def publishedItems(self, subpath=None, exact = None):
        """
          Return the list of the published items
        """
        if subpath is None:
            subpath = self.REQUEST.get('traverse_subpath', [])
            path = self.internal + '/' + join(subpath, '/')
            if not self.listPublications(path):
               subpath = subpath[:-1]
	
        subpath = join(subpath, '/')
        myPath = self.getSiteUrl() + '/' + subpath

        return self.listPublications( myPath, exact )

    security.declarePublic('boxView')
    def boxView(self, REQUEST=None):
        """\
           Box view
        """
        if REQUEST is not None:
           return self.listBoxForm(self, REQUEST)

    def getPublisher(self):
	"""
	Return context of publisher (to get site root in subdocuments)
	"""
	return self

    def isSiteObject(self, path):
        """
        Return 1 if the object is located inside 'custom'
        """
        try:
	    self.custom.unrestrictedTraverse(path)
	except AttributeError:
	    return 0
	except KeyError:
            return 0
        return 1

    security.declarePublic('getSite')
    def getSite(self):
        """
          Return the internal site object
        """
        site_url = self.getSiteUrl()
        site = self.unrestrictedTraverse(site_url, None)
        return site.aq_parent

    security.declarePublic('getSiteUrl')
    def getSiteUrl(self):
        """
          Return the path to the internal site
        """
        if self.internal == 'self':
            return join(self.portal_url.getPortalObject().getPhysicalPath(), '/')
	return self.internal
              	
    def getParents(self, subpath=None):
        """
          Return the list of item's containers
        """
        parents = []

        if subpath is None:
            subpath = self.REQUEST.get('traverse_subpath', [])

	l = len(subpath)
	level = 1
        while len( subpath[:] ):
           path = self.getSiteUrl() + '/' + join(subpath, '/')
           parent = self.unrestrictedTraverse( path, None )
           if parent is None:
              subpath = subpath[:-1]
              continue

           if parent.title or level == l:
              parents.insert(0, {'absolute_url': parent.external_url(),
                                 'meta_type'   : parent.meta_type,
                                 'title_or_id' : parent.title_or_id(),
                                 'title'       : parent.title,
                                 'id'          : parent.getId(),
                                })

           subpath = subpath[:-1]
	   level+=1
        return parents

    security.declareProtected(CMFCorePermissions.View, 'actions')
    def actions(self, subpath = None):
        """
          Return folders list
        """
        if subpath is None:
            subpath = self.REQUEST.get('traverse_subpath', [])
            path = self.internal + '/' + join(subpath, '/')
            if not self.listPublications(path):
               subpath = subpath[:-1]


        if type(subpath) == type([]):
           subpath = join(subpath, '/')

        internal_folder = self.unrestrictedTraverse( self.getSiteUrl() + '/' + subpath, None )

        if internal_folder is None:
            return

        subfolders=[]
        catalog = getToolByName(self.content, 'portal_catalog')
        for item in internal_folder.objectValues(['Heading',]):
             # Filter out folders with no published
             # documents inside
             if catalog.searchResults( meta_type = 'HTMLDocument'
                                     , review_state = 'published'
                                     , path = item.absolute_url(1)
                                     ):
                  subfolders.append(item)

        # Do not return object instances, we need only hash
        # with some object information
        actions_list = map(lambda x: {'absolute_url': x.external_url(),
                                      'title'       : x.title,
                                      'title_or_id' : x.title_or_id(),
                                      'Description' : x.Description(),
                                      'id'          : x.getId()
                                     },
                           subfolders)

        return actions_list

    security.declareProtected( CMFCorePermissions.ManagePortal, 'editProperties' )
    def editProperties( self, internal=None, title=None, REQUEST=None):
        """
            Form handler for the publisher settings
        """
        self.title = title
        msg = "Tool updated."
        if internal is not None:
            if internal == 'self':
                  self.internal = internal               
            else:
               try:
                  internal_site = self.restrictedTraverse( internal )
##                  if internal_site.meta_type != 'CMF Site':
##                      raise ValueError, "%s is not a valid CMF Site" % internal

                  if not hasattr(internal_site, 'publishers'):
                     internal_site.manage_addProperty('publishers', '', 'lines')
     
                  publishers = internal_site.publishers
                  my_url = join(self.getPhysicalPath(), '/')
                  if my_url not in tuple(publishers):
                     publishers.append(my_url)
                     internal_site.manage_changeProperties(publishers=publishers)
                    
                  self.internal = internal
               except:
                  msg = "%s is not a valid CMF Site" % internal

        if REQUEST is not None:
            return self.manage_main( self, REQUEST, manage_tabs_message = msg )

    security.declarePrivate( 'fetchObject' )
    def fetchObject( self, path, REQUEST=None):
        """
            Fetch the published object from the internal site
        """
        base_path = path.split('/')

        # Do not call useless methods (return content instead)
        if base_path[-1] in ['view', ]: 
           base_path = base_path[:-1]

        o_id = base_path[-1]
        if hasattr(self, o_id):
           # Check the external portal for the object
           local_object = self.unrestrictedTraverse( o_id )
           if local_object.meta_type in ['DTMLDocument', ]:
              return local_object(self, REQUEST)
           else:
              return local_object.index_html(REQUEST, REQUEST['RESPONSE'])
        else:
           # Check the internal portal for the object
           internal_object = self.unrestrictedTraverse( join( base_path, '/'), None )
           if internal_object.meta_type in ['Portal Folder', 'Folder', 'Heading']:
              # Return the main topic document if necessary
              if internal_object.hasMainPage():
                 index_url = internal_object.getMainPage().absolute_url(1)
                 return self.fetchObject(index_url, REQUEST)
              # Display documents list otherwise
              return self.listPublishedForm(self, REQUEST)

           parents = []
           parent = internal_object
           while(1):
             if parent.id != 'storage':
                parents.append( parent.aq_parent )
	        parent = parent.aq_parent
             else:
                break

           # Add requested object in context of the
           # external CMF Site
           slot = self
           while len(parents):
              slot.ob = parents.pop()
              slot = slot.ob
              slot.ob = internal_object

           # Found something 
           if hasattr(slot.ob.aq_self, 'text'):		    
              text = slot.ob.text
              text = text.replace( self.getSiteUrl(), '/' + self.absolute_url(1) )
              slot.ob.text = text

              return slot.ob()
  
           if internal_object.meta_type in ['DTMLDocument', ]:
              return slot.ob(slot.ob, REQUEST)
	  
           return slot.ob.index_html(REQUEST, REQUEST['RESPONSE'])

    security.declarePublic('subscription')
    def subscription(self, REQUEST=None):
        """
           Interface between external site users and heading
           subscription methods
        """
        r = REQUEST.get
        email = r('email', None)
        secret_code = r('secret_code', None)

        # where we are?
        subpath = r('subpath', None) or r('traverse_subpath', [])
        if type(subpath) == type([]): 
             subpath = join(subpath, '/')

        action=r('action', 'subscribe')

	result = 'Got empty email'
        path = self.getSiteUrl() + '/' + subpath
        ob = self.unrestrictedTraverse( path, None )
        if ob is not None:
            if ob.meta_type != 'Heading':
               ob = ob.aq_parent

            if email:
               if action == 'subscribe':
                  result = ob.subscribe(email, REQUEST)
               elif action == 'unsubscribe':
                  result = ob.unsubscribe(email, REQUEST)
               elif action == 'confirm' and secret_code:
                  result = ob.confirm_subscription(email, secret_code, REQUEST)

	if REQUEST is not None:
	    REQUEST['RESPONSE'].redirect("%s/%s?portal_status_message=%s" % (self.absolute_url(), subpath, result))

InitializeClass(NauPublisher)

manage_addNauPublisherForm = DTMLFile('dtml/addNauPublisher', globals() )

def manage_addNauPublisher(self, id, title='', internal='', REQUEST=None):
    """\
       Add a new Publisher object with id *id*.
    """
    ob=NauPublisher(id, title, internal)
    self._setObject(id, ob)
    if REQUEST is not None:
        REQUEST['RESPONSE'].redirect(self.absolute_url() + '/manage_main')

