"""NauSite class
$Id: NauSite.py,v 1.5 2002/09/25 14:41:45 ikuleshov Exp $
"""
__version__='$Revision: 1.5 $'[11:-2]
 
import Globals
from AccessControl.Permissions import view, access_contents_information, delete_objects
from Products.CMFCore.CMFCorePermissions import RequestReview, \
                                                ModifyPortalContent, \
                                                ReviewPortalContent, \
                                                ManagePortal, \
                                                ListFolderContents, \
                                                AddPortalContent, \
                                                AddPortalFolders, \
                                                ChangePermissions

from Products.CMFCore import PortalFolder
from Products.CMFCore.PortalObject import PortalObjectBase
from Products.CMFCore.CatalogTool import CatalogTool
from Products.CMFCore.TypesTool import ContentFactoryMetadata
from Products.CMFCore.utils import getToolByName
from Products.CMFDefault.DublinCore import DefaultDublinCoreImpl

from Products.CMFDefault import Document, Image, File, Link, NewsItem, Favorite
from Products.CMFDefault import DiscussionItem, SkinnedFolder
from Products.CMFDefault import cmfdefault_globals

from Products.CMFCore.WorkflowTool import addWorkflowFactory
from Products.DCWorkflow.DCWorkflow import DCWorkflowDefinition

import NauBoardItem, NauBoardContainer
import HTMLDocument, DTMLDocument, Vote, Heading, TaskItem, Report, Site, SiteImage
import NauPublishTool

from urllib import quote
from string import split, join
import os

factory_type_information = ( NauBoardItem.factory_type_information 
                           + NauBoardContainer.factory_type_information
			   + HTMLDocument.factory_type_information
			   + DTMLDocument.factory_type_information
			   + Vote.factory_type_information
			   + Heading.factory_type_information
			   + NauPublishTool.factory_type_information
			   + TaskItem.factory_type_information
			   + Report.factory_type_information
                           + Site.factory_type_information
                           + SiteImage.factory_type_information
                           )

cmf_factory_type_information = ( DiscussionItem.factory_type_information
                               + SkinnedFolder.factory_type_information
                               )

class NauSite ( PortalObjectBase
              , DefaultDublinCoreImpl
              ):
    """
        The *only* function this class should have is to help in the setup
        of a new NauSite.  It should not assist in the functionality at all.
    """
    meta_type = 'NauSite'

    _properties = (
        {'id':'title', 'type':'string'},
        {'id':'description', 'type':'text'},
        )
    title = ''
    description = ''
    test = {}

    __ac_permissions__=( ( 'Manage portal', ('manage_migrate_content',) )
                       , ( 'Add portal content', () )
                       , ( 'Add portal folders', () )
                       , ( 'List portal members', () )
                       , ( 'Reply to item', () )
                       , ( 'View',  ( 'isEffective', ) )
                       )

    def __init__( self, id, title='' ):
        PortalObjectBase.__init__( self, id, title )
        DefaultDublinCoreImpl.__init__( self )

    def isEffective( self, date ):
        """
            Override DefaultDublinCoreImpl's test, since we are always viewable.
        """
        return 1

    def reindexObject( self ):
        """
            Override DefaultDublinCoreImpl's method (so that we can play
            in 'editMetadata').
        """
        pass

    def absolute_url(self, relative=0):
        try:
            req = self.REQUEST
        except AttributeError:
            req = {}
        rpp = req.get('VirtualRootPhysicalPath', ('',))
        spp = self.getPhysicalPath()
        i = 0
        for name in rpp[:len(spp)]:
            if spp[i] == name:
                i = i + 1
            else:
                break
        path = map(quote, spp[i:])
        # This is useful for physical path relative to a VirtualRoot
        if relative:
             return join(path, '/')
        else:
             return '/' + join(path, '/')

Globals.InitializeClass(NauSite)

# This class is based on Products.CMFDefault.Portal.PortalGenerator class
class PortalGenerator:

    klass = NauSite

    def setupTools(self, p):
        """Set up initial tools"""

        addCMFCoreTool = p.manage_addProduct['CMFCore'].manage_addTool
        addCMFCoreTool('CMF Actions Tool', None)
        addCMFCoreTool('CMF Catalog', None)
        addCMFCoreTool('CMF Member Data Tool', None)
        addCMFCoreTool('CMF Skins Tool', None)
        addCMFCoreTool('CMF Types Tool', None)
        addCMFCoreTool('CMF Undo Tool', None)
        addCMFCoreTool('CMF Workflow Tool', None)

        addCMFDefaultTool = p.manage_addProduct['CMFDefault'].manage_addTool
        addCMFDefaultTool('Default Discussion Tool', None)
        addCMFDefaultTool('Default Membership Tool', None)
        addCMFDefaultTool('Default Registration Tool', None)
        addCMFDefaultTool('Default URL Tool', None)
        addCMFDefaultTool('Default Properties Tool', None)
        addCMFDefaultTool('Default Syndication Tool', None)

        addNauSiteTool = p.manage_addProduct['CMFNauTools'].manage_addTool
        addNauSiteTool('NauSite Publisher Tool', None)
        addNauSiteTool('NauSite User Groups Tool', None)
        addNauSiteTool('NauSite Metadata Tool', None)
        addNauSiteTool('NauSite Help Tool', None)

    def setupMessageCatalog(self, p):
        p.manage_addProduct['Localizer'].manage_addMessageCatalog(
            'msg', 'Messages', ['en', 'ru', 'de'])

        def import_locale(p, path, language):
            """
              Import PO file into the Message Catalog
            """
            open(path + '/%s.po' % language, 'rt')
            try: f = open(path + '/%s.po' % language, 'rt')
            except: pass
            else:
              p.msg.manage_import(language, f)

        path=os.path.join(Globals.package_home(globals()),
                         'locale')

        import_locale(p, path, 'en')
        import_locale(p, path, 'ru')
        import_locale(p, path, 'de')

    def setupMailHost(self, p):
        p.manage_addProduct['MailHost'].manage_addMailHost(
            'MailHost', smtp_host='localhost')

    def setupUserFolder(self, p):
        p.manage_addProduct['NuxUserGroups'].addUserFolderWithGroups()

    def setupCookieAuth(self, p):
        p.manage_addProduct['CMFCore'].manage_addCC(
            id='cookie_authentication')
        p.cookie_authentication.auto_login_page = ''

    def setupMembersFolder(self, p):
        PortalFolder.manage_addPortalFolder(p, 'Members')
        p.Members.manage_addProduct['OFSP'].manage_addDTMLMethod(
            'index_html', 'Member list', '<dtml-return roster>')

    def setupRoles(self, p):
        # Set up the suggested roles.
        p.__ac_roles__ = ('Member', 'Reviewer', 'Editor', 'Writer', 'Reader', 'Author')

    def setupPermissions(self, p):
        # Set up some suggested roles to permission mappings.
        mp = p.manage_permission

        mp('Set own password',        ['Member','Manager',],    1)
        mp('Set own properties',      ['Member','Manager',],    1)
        mp('List undoable changes',   ['Member','Manager',],    1)
        mp('Add portal content',      ['Owner','Manager', 'Editor', 'Writer', 'Author'],     1)
        mp('Add portal folders',      ['Owner','Manager', 'Editor', 'Writer'],     1)
        mp('Review portal content',   ['Reviewer','Manager',],  1)
        mp('Access future portal content',
                                      ['Reviewer','Manager',],  1)
        mp('List portal members',     ['Member','Manager',],    1)
        mp('Reply to item',           ['Member','Manager',],    1)

        # Add some other permissions mappings that may be helpful.
        mp('Delete objects',          ['Owner','Manager', 'Editor'],     1)
        mp('FTP access',              ['Owner','Manager',],     1)
        mp('Manage properties',       ['Owner','Manager',],     1)
        mp('Modify portal content',   ['Owner','Manager',],     1)
        mp('Change permissions',      ['Owner','Manager', 'Editor'],     1)
        mp('View management screens', ['Owner','Manager', 'Editor', 'Writer', 'Reader', 'Author'],     1)
        mp('Undo changes',            ['Member','Manager',],     1)
        mp('Request review',          ['Member','Manager',],     1)
        mp('View',                    ['Member','Manager',],     0)

    def setupDefaultSkins(self, p):
        from Products.CMFCore.DirectoryView import addDirectoryViews
        ps = getToolByName(p, 'portal_skins')
        addDirectoryViews(ps, 'skins', globals())
        addDirectoryViews(ps, 'skins', cmfdefault_globals)
        ps.manage_addProduct['OFSP'].manage_addFolder(id='custom')
        ps.addSkinSelection('Site',
            'custom, gui, mail_templates, common, groups, htmldocument, heading,'
            + 'vote, nauboard, discussions, tracker,'
            + 'content, generic, control, Images',
            make_default=1)
        p.setupCurrentSkin()

    def setupTypes(self, p, initial_types=factory_type_information):
        tool = getToolByName(p, 'portal_types', None)
        if tool is None:
            return
        for t in initial_types:
            cfm = apply(ContentFactoryMetadata, (), t)
            tool._setObject(t['id'], cfm)
        
    def setupMimetypes(self, p):
        p.manage_addProduct[ 'CMFCore' ].manage_addRegistry()
        reg = p.content_type_registry

        reg.addPredicate( 'dtml', 'extension' )
        reg.getPredicate( 'dtml' ).edit( extensions="dtml" )
        reg.assignTypeName( 'dtml', 'DTMLDocument' )

        reg.addPredicate( 'link', 'extension' )
        reg.getPredicate( 'link' ).edit( extensions="url, link" )
        reg.assignTypeName( 'link', 'Link' )

        reg.addPredicate( 'news', 'extension' )
        reg.getPredicate( 'news' ).edit( extensions="news" )
        reg.assignTypeName( 'news', 'News Item' )

        reg.addPredicate( 'document', 'major_minor' )
        reg.getPredicate( 'document' ).edit( major="text", minor="" )
        reg.assignTypeName( 'document', 'HTMLDocument' )

        reg.addPredicate( 'image', 'major_minor' )
        reg.getPredicate( 'image' ).edit( major="image", minor="" )
        reg.assignTypeName( 'image', 'Site Image' )

        reg.addPredicate( 'file', 'major_minor' )
        reg.getPredicate( 'file' ).edit( major="application", minor="" )
        reg.assignTypeName( 'file', 'File' )

    def setupWorkflow(self, p):
        tool = getToolByName(p, 'portal_workflow', None)
        if tool is None:
            return

        id = 'document_workflow'
        ob = DCWorkflowDefinition(id)
        setupDocumentWorkflow(ob)
        tool._setObject(id, ob)

        id = 'heading_workflow'
        ob = DCWorkflowDefinition(id)
        setupHeadingWorkflow(ob)
        tool._setObject(id, ob)

        tool.setChainForPortalTypes( ( 'HTMLDocument', 'DTMLDocument', 'Site Image'), ('document_workflow', ) )
        tool.setChainForPortalTypes( ( 'Heading', ), ('heading_workflow', ) )
        tool.setDefaultChain('')

    def setupStorage(self, p, home_title=''):
        p.manage_addProduct['CMFNauTools'].manage_addHeading(id='storage', title='Content storage')
        p.manage_addProduct['CMFNauTools'].manage_addHeading(id='external', title='Sites repository')
        p.manage_addProduct['CMFNauTools'].manage_addHeading(id='plugins', title='Site plugins')

        # Add external methods to enable Site plugins. 
        # TODO: remove this in future. Installer must not be aware of plugins.
        plugins = p.plugins 
        try:
           plugins.manage_addProduct['ExternalMethod'].manage_addExternalMethod('NauForum', 'NauForum plugin', 'NauForum.deploy', 'deploy_NauForum')
        except:
           pass

        try:
           plugins.manage_addProduct['ExternalMethod'].manage_addExternalMethod('Tracker', 'Tracker plugin', 'Tracker.deploy', 'deploy_Tracker')
        except:
           pass

        p.storage.manage_addProduct['CMFNauTools'].manage_addHeading(id='members', title=home_title)
        p.storage.members.manage_addProduct['CMFNauTools'].manage_addHeading(id='user_defaults')

        mp = p.storage.manage_permission
        mp('List folder contents', ['Owner','Manager', 'Editor', 'Writer', 'Reader', 'Author'],     0)

        tool = getToolByName(p, 'portal_groups', None)
        if tool is None:
            return
        tool._addGroup('All users')


    def setupTracker(self, p):
        p.manage_addProduct['Tracker'].manage_tracker_add()

    def setupActions(self, p):
        tool = getToolByName(p, 'portal_actions', None)
        if tool is None:
            return

        tool.action_providers =  ( 'portal_registration'
                                 , 'portal_discussion'
                                 , 'portal_undo'
                                 , 'portal_workflow'
                                 , 'portal_properties'
                                 , 'portal_groups'
                                 , 'portal_publisher'
                                 , 'portal_metadata'
                                 , 'portal_help'
                                 )

    def setupMemberData(self, p):
        tool = getToolByName(p, 'portal_memberdata', None)
        if tool is None:
            return

        tool._setProperty('fname', '', 'string')
        tool._setProperty('mname', '', 'string')
        tool._setProperty('lname', '', 'string')
        tool._setProperty('company', '', 'string')
        tool._setProperty('notes', '', 'text')
        tool._setProperty('phone', '', 'string')
        tool._setProperty('rate_sum', 0, 'int')
        tool._setProperty('rate_users', '', 'tokens')

    def setup(self, p, create_userfolder, home_title):
        self.setupTools(p)
        self.setupMemberData(p)
        self.setupMessageCatalog(p)
        self.setupMailHost(p)
        if int(create_userfolder) != 0:
            self.setupUserFolder(p)
        self.setupCookieAuth(p)
        self.setupMembersFolder(p)
        self.setupRoles(p)
        self.setupPermissions(p)
        self.setupDefaultSkins(p)

        #   SkinnedFolders are only for customization;
        #   they aren't a default type.
        default_types = tuple( filter( lambda x: x['id'] != 'Skinned Folder'
                                     , factory_type_information ) )
        self.setupTypes(p, default_types )
        self.setupTypes(p, cmf_factory_type_information )

        self.setupTypes(p, PortalFolder.factory_type_information)
        self.setupStorage(p, home_title)
        self.setupMimetypes(p)
        self.setupWorkflow(p)
        self.setupTracker(p)
        self.setupActions(p)

    def create(self, parent, id, create_userfolder, home_title):
        id = str(id)
        portal = self.klass(id=id)
        parent._setObject(id, portal)
        # Return the fully wrapped object.
        p = parent.this()._getOb(id)
        self.setup(p, create_userfolder, home_title)
        return p

    def setupDefaultProperties(self, p, title, description,
                               email_from_address, email_from_name,
                               validate_email,
                               ):
        p._setProperty('email_from_address', email_from_address, 'string')
        p._setProperty('email_from_name', email_from_name, 'string')
        p._setProperty('validate_email', validate_email and 1 or 0, 'boolean')
        p.title = title
        p.description = description


manage_addNauSiteForm = Globals.HTMLFile('dtml/addNauSite', globals())
manage_addNauSiteForm.__name__ = 'addNauSite'

def manage_addNauSite(self, id='docs', title='NauSite', description='',
                         home_title='',
                         create_userfolder=1,
                         email_from_address='postmaster@localhost',
                         email_from_name='NauSite Administrator',
                         validate_email=0, RESPONSE=None):
    """
    Adds NauSIte instance.
    """
    gen = PortalGenerator()
    from string import strip
    id = strip(id)
    p = gen.create(self, id, create_userfolder, home_title)
    gen.setupDefaultProperties(p, title, description,
                               email_from_address, email_from_name,
                               validate_email)
    if RESPONSE is not None:
        RESPONSE.redirect(p.absolute_url() + '/finish_nausite_construction')


p_access = access_contents_information
p_delete = delete_objects
p_list = ListFolderContents
p_modify = ModifyPortalContent
p_review = ReviewPortalContent
p_view = view
p_manage = ManagePortal
p_addcontent = AddPortalContent
p_addfolders = AddPortalFolders
p_changeperms = ChangePermissions
p_request = RequestReview

r_anon = 'Anonymous'
r_manager = 'Manager'
r_reviewer = 'Reviewer'
r_owner = 'Owner'
r_member = 'Member'
r_editor = 'Editor'
r_reader = 'Reader'
r_writer = 'Writer'
r_author = 'Author'

def setupDocumentWorkflow(wf):
    """
      Setup the default NauSite documents workflow
    """
    wf.setProperties(title='NauSite document workflow')

    for s in ('private', 'private_rejected', 'evolutive', 'group', 'fixed',
              'pending', 'fixed_reviewed', 'awaiting_publication', 'published'
             ):
        wf.states.addState(s)

    for t in ('retract', 'evolve', 'open_for_group', 'fix',
              'submit', 'refuse', 'confirm', 'fix_reviewed',
              'request_publication', 'publish'
             ):
        wf.transitions.addTransition(t)

    for v in ('action', 'actor', 'comments', 'review_history', 'time'):
        wf.variables.addVariable(v)
    for l in ('publisher_queue',):
        wf.worklists.addWorklist(l)
    for p in (p_access, p_delete, p_list, p_modify, p_review, p_view):
        wf.addManagedPermission(p)

    #
    # Setup states
    wf.states.setInitialState('evolutive')

    sdef = wf.states['private']
    sdef.setProperties(
        title='Object is only visible to it\'s owner',
        transitions=('evolve', 'fix', 'open_for_group', 'submit', 'request_publication', 'publish'))
    sdef.setPermission(p_access, 0, (r_manager, r_owner))
    sdef.setPermission(p_delete, 1, ())
    sdef.setPermission(p_list, 1, ())
    sdef.setPermission(p_modify, 0, (r_manager, r_owner))
    sdef.setPermission(p_review, 1, ())
    sdef.setPermission(p_view, 0, (r_manager, r_owner))

    sdef = wf.states['private_rejected']
    sdef.setProperties(
        title='Object is no longer editable and only visible to it\'s owner',
        transitions=())
    sdef.setPermission(p_access, 0, (r_manager, r_owner))
    sdef.setPermission(p_delete, 1, ())
    sdef.setPermission(p_list, 1, ())
    sdef.setPermission(p_modify, 0, ())
    sdef.setPermission(p_review, 0, ())
    sdef.setPermission(p_view, 0, (r_manager, r_owner))

    sdef = wf.states['evolutive']
    sdef.setProperties(
        title='Object is visible to everyone with enough permissions to access contents of it\s folder',
        transitions=('retract', 'submit', 'fix', 'open_for_group', 'request_publication', 'publish'))
    sdef.setPermission(p_access, 1, ())
    sdef.setPermission(p_delete, 1, ())
    sdef.setPermission(p_list, 1, ())
    sdef.setPermission(p_modify, 1, ())
    sdef.setPermission(p_review, 0, ())
    sdef.setPermission(p_view, 0, (r_manager, r_owner, r_editor, r_reader, r_reviewer, r_writer))

    sdef = wf.states['group']
    sdef.setProperties(
        title='Object is editable by everyone with writer\'s permissions in it\s folder',
        transitions=('retract', 'evolve', 'submit', 'fix', 'request_publication', 'publish'))
    sdef.setPermission(p_access, 1, ())
    sdef.setPermission(p_delete, 1, ())
    sdef.setPermission(p_list, 1, ())
    sdef.setPermission(p_modify, 1, (r_manager, r_owner, r_editor, r_writer))
    sdef.setPermission(p_review, 0, ())
    sdef.setPermission(p_view, 0, (r_manager, r_owner, r_editor, r_reader, r_reviewer, r_writer))

    sdef = wf.states['fixed']
    sdef.setProperties(
        title='Object is not editable and removable',
        transitions=('publish', 'request_publication'))
    sdef.setPermission(p_access, 1, ())
    sdef.setPermission(p_delete, 0, ())
    sdef.setPermission(p_list, 1, ())
    sdef.setPermission(p_modify, 0, ())
    sdef.setPermission(p_review, 0, (r_owner,))
    sdef.setPermission(p_view, 0, (r_manager, r_owner, r_editor, r_reader, r_reviewer, r_writer))

    sdef = wf.states['pending']
    sdef.setProperties(
        title='Object is pending for review',
        transitions=('refuse', 'confirm', 'fix_reviewed', 'retract', 'evolve', 'open_for_group'))
    sdef.setPermission(p_access, 1, (r_anon, r_manager, r_reviewer))
    sdef.setPermission(p_delete, 1, ())
    sdef.setPermission(p_list, 1, ())
    sdef.setPermission(p_modify, 0, (r_manager, r_reviewer))
    sdef.setPermission(p_review, 0, (r_owner, r_reviewer))
    sdef.setPermission(p_view, 0, (r_manager, r_owner, r_editor, r_reader, r_reviewer, r_writer))

    sdef = wf.states['fixed_reviewed']
    sdef.setProperties(
        title='Object is reviewed',
        transitions=('publish', 'request_publication'))
    sdef.setPermission(p_access, 1, ())
    sdef.setPermission(p_delete, 0, (r_editor, r_manager))
    sdef.setPermission(p_list, 1, ())
    sdef.setPermission(p_modify, 0, ())
    sdef.setPermission(p_review, 0, ())
    sdef.setPermission(p_view, 0, (r_manager, r_owner, r_editor, r_reader, r_reviewer, r_writer))

    sdef = wf.states['awaiting_publication']
    sdef.setProperties(
        title='Object is waiting to be published on the external site',
        transitions=('retract', 'open_for_group', 'evolve', 'fix', 'publish'))
    sdef.setPermission(p_access, 1, ())
    sdef.setPermission(p_delete, 1, ())
    sdef.setPermission(p_list, 1, ())
    sdef.setPermission(p_modify, 0, ())
    sdef.setPermission(p_review, 1, ())
    sdef.setPermission(p_view, 0, (r_manager, r_owner, r_editor, r_reader, r_reviewer, r_writer))

    sdef = wf.states['published']
    sdef.setProperties(
        title='Object is published on the external site',
        transitions=('retract', 'evolve', 'open_for_group', 'fix'))
    sdef.setPermission(p_access, 1, (r_anon, r_manager, r_member))
    sdef.setPermission(p_delete, 1, ())
    sdef.setPermission(p_list, 1, (r_member,))
    sdef.setPermission(p_modify, 0, ())
    sdef.setPermission(p_review, 0, (r_owner,))
    sdef.setPermission(p_view, 1, (r_anon, r_manager, r_member))

    #
    # Set transactions
    tdef = wf.transitions['retract']
    tdef.setProperties(
        title='Hide document',
        new_state_id='private',
        actbox_name='Retract',
        actbox_url='%(content_url)s/document_retract_form',
        props={'guard_roles':r_owner, 
               'guard_permissions':p_request
              }
       )

    tdef = wf.transitions['evolve']
    tdef.setProperties(
        title='Evolve document',
        new_state_id='evolutive',
        actbox_name='Evolve',
        actbox_url='%(content_url)s/document_evolve_form',
        props={'guard_roles':r_owner, 
               'guard_permissions':p_request
              }
       )

    tdef = wf.transitions['open_for_group']
    tdef.setProperties(
        title='Open the document for group editing',
        new_state_id='group',
        actbox_name='Open for group',
        actbox_url='%(content_url)s/document_group_form',
        props={'guard_roles':r_owner, 
               'guard_permissions':p_request
              }
       )

    tdef = wf.transitions['fix']
    tdef.setProperties(
        title='Fix the document',
        new_state_id='fixed',
        actbox_name='Fix',
        actbox_url='%(content_url)s/document_fix_form',
        props={'guard_roles':r_owner, 
               'guard_permissions':p_request
              }
       )

    tdef = wf.transitions['submit']
    tdef.setProperties(
        title='Submit the document for review',
        new_state_id='pending',
        props={'guard_roles':r_owner, 
               'guard_permissions':p_request
              }
       )

    tdef = wf.transitions['refuse']
    tdef.setProperties(
        title='Refuse to review the document',
        new_state_id='private_rejected',
        props={'guard_roles':r_owner, 
               'guard_permissions':p_request
              }
       )

    tdef = wf.transitions['confirm']
    tdef.setProperties(
        title='Review the document',
        new_state_id='pending',
        actbox_name='Confirm',
        actbox_url='%(content_url)s/confirm',
        props={'guard_expr':'python: here.portal_membership.getAuthenticatedMember().getUserName() in here.requested_users', 
               'guard_permissions':p_request
              }
       )

    ## TODO: Check and test security issues
    tdef = wf.transitions['fix_reviewed']
    tdef.setProperties(
        title='Fix the reviewed document',
        new_state_id='fixed_reviewed',
       )

    tdef = wf.transitions['request_publication']
    tdef.setProperties(
        title='Request the document publication on the external site',
        new_state_id='awaiting_publication',
        actbox_name='Request publication',
        actbox_url='%(content_url)s/document_request_publication_form',
        props={'guard_roles':r_owner, 
               'guard_permissions':p_request,
               'guard_expr':'python: here.getSite() is not None'
              }
       )

    tdef = wf.transitions['publish']
    tdef.setProperties(
        title='Publish the document on the external site',
        new_state_id='published',
        actbox_name='Publish',
        actbox_url='%(content_url)s/document_publish_form',
        props={'guard_permissions':p_manage,
               'guard_expr':'python: here.getSite() is not None'
              }
       )

    wf.variables.setStateVar('review_state')

    vdef = wf.variables['action']
    vdef.setProperties(description='The last transition',
                       default_expr='transition/getId|nothing',
                       for_status=1, update_always=1)

    vdef = wf.variables['actor']
    vdef.setProperties(description='The ID of the user who performed '
                       'the last transition',
                       default_expr='user/getUserName',
                       for_status=1, update_always=1)

    vdef = wf.variables['comments']
    vdef.setProperties(description='Comments about the last transition',
                       default_expr="python:state_change.kwargs.get('comment', '')",
                       for_status=1, update_always=1)

    vdef = wf.variables['review_history']
    vdef.setProperties(description='Provides access to workflow history',
                       default_expr="state_change/getHistory",
                       props={'guard_permissions':
                              p_request + ';' + p_review})

    vdef = wf.variables['time']
    vdef.setProperties(description='Time of the last transition',
                       default_expr="state_change/getDateTime",
                       for_status=1, update_always=1)

    ldef = wf.worklists['publisher_queue']
    ldef.setProperties(description='Publisher tasks',
                       actbox_name='Pending for publication',
                       actbox_url='%(portal_url)s/search?review_state=awaiting_publication',
                       props={'var_match_review_state':'awaiting_publication',
                              'guard_permissions':p_manage})

def setupHeadingWorkflow(wf):
    """
      Setup the default NauSite headings workflow
    """
    wf.setProperties(title='NauSite heading workflow')

    for s in ('editable', 'frozen', 'fixed'):
        wf.states.addState(s)

    for t in ('edit', 'freeze', 'fix'):
        wf.transitions.addTransition(t)

    for v in ('action', 'actor', 'comments', 'review_history', 'time'):
        wf.variables.addVariable(v)

    for p in (p_addcontent, p_addfolders, p_changeperms, p_delete, p_modify):
        wf.addManagedPermission(p)

    #
    # Setup states
    wf.states.setInitialState('editable')

    sdef = wf.states['editable']
    sdef.setProperties(
        title='Editable',
        transitions=('fix', 'freeze'))
    sdef.setPermission(p_addcontent, 1, ())
    sdef.setPermission(p_addfolders, 1, ())
    sdef.setPermission(p_changeperms, 1, ())
    sdef.setPermission(p_delete, 1, ())
    sdef.setPermission(p_modify, 1, ())

    sdef = wf.states['frozen']
    sdef.setProperties(
        title='Frozen',
        transitions=('edit', 'fix'))
    sdef.setPermission(p_addcontent, 0, ())
    sdef.setPermission(p_addfolders, 0, ())
    sdef.setPermission(p_changeperms, 1, ())
    sdef.setPermission(p_delete, 0, ())
    sdef.setPermission(p_modify, 0, ())

    sdef = wf.states['fixed']
    sdef.setProperties(
        title='Fixed',
        transitions=())
    sdef.setPermission(p_addcontent, 0, ())
    sdef.setPermission(p_addfolders, 0, ())
    sdef.setPermission(p_changeperms, 0, ())
    sdef.setPermission(p_delete, 0, ())
    sdef.setPermission(p_modify, 0, ())

    #
    # Set transactions
    tdef = wf.transitions['edit']
    tdef.setProperties(
        title='Edit folder',
        new_state_id='editable',
        actbox_name='Edit',
        actbox_url='%(content_url)s/heading_edit_form',
        props={ 'guard_permissions':p_changeperms }
       )

    tdef = wf.transitions['freeze']
    tdef.setProperties(
        title='Freeze folder',
        new_state_id='frozen',
        actbox_name='Freeze',
        actbox_url='%(content_url)s/heading_freeze_form',
        props={ 'guard_permissions':p_changeperms }
       )

    tdef = wf.transitions['fix']
    tdef.setProperties(
        title='Fix folder',
        new_state_id='fixed',
        actbox_name='Fix',
        actbox_url='%(content_url)s/heading_fix_form',
        props={ 'guard_permissions':p_changeperms }
       )


    wf.variables.setStateVar('review_state')

    vdef = wf.variables['action']
    vdef.setProperties(description='The last transition',
                       default_expr='transition/getId|nothing',
                       for_status=1, update_always=1)

    vdef = wf.variables['actor']
    vdef.setProperties(description='The ID of the user who performed '
                       'the last transition',
                       default_expr='user/getUserName',
                       for_status=1, update_always=1)

    vdef = wf.variables['comments']
    vdef.setProperties(description='Comments about the last transition',
                       default_expr="python:state_change.kwargs.get('comment', '')",
                       for_status=1, update_always=1)

    vdef = wf.variables['review_history']
    vdef.setProperties(description='Provides access to workflow history',
                       default_expr="state_change/getHistory",
                       props={'guard_permissions':
                              p_request + ';' + p_review})

    vdef = wf.variables['time']
    vdef.setProperties(description='Time of the last transition',
                       default_expr="state_change/getDateTime",
                       for_status=1, update_always=1)


# Redefine portal_catalog initial indexes
def enumerateIndexes( self ):
    #   Return a list of ( index_name, type ) pairs for the initial
    #   index set.
    return ( ('Title', 'TextIndex')
           , ('Subject', 'KeywordIndex')
           , ('Description', 'TextIndex')
           , ('Creator', 'FieldIndex')
           , ('SearchableText', 'TextIndex')
           , ('Date', 'FieldIndex')
           , ('Type', 'FieldIndex')
           , ('created', 'FieldIndex')
           , ('effective', 'FieldIndex')
           , ('expires', 'FieldIndex')
           , ('modified', 'FieldIndex')
           , ('allowedRolesAndUsers', 'KeywordIndex')
           , ('review_state', 'FieldIndex')
           , ('in_reply_to', 'FieldIndex')
           , ('parent_url', 'FieldIndex')
           )
CatalogTool.enumerateIndexes = enumerateIndexes
CatalogTool.enumerateIndexes__roles__ = None # public
