"""Report content class.

$Id: Report.py,v 1.8 2002/09/19 04:20:20 ikuleshov Exp $
"""
import re, string, os.path
import Globals

from Acquisition import aq_base
from OFS.Folder import Folder
from AccessControl import ClassSecurityInfo
from DateTime import DateTime
from string import join

from Products.CMFDefault.DublinCore import DefaultDublinCoreImpl
from Products.CMFDefault.Document import Document
from Products.CMFCore.WorkflowCore import WorkflowAction
from Products.CMFCore.utils import getToolByName, _checkPermission, _getViewFor
from Products.CMFCore import CMFCorePermissions
from Products.CMFCore.CMFCatalogAware import CMFCatalogAware
from Products.Photo import Photo

from SimpleNauItem import SimpleNauItem
from ManageCMFContent import ManageCMFContent

factory_type_information = ( { 'id'             : 'Report'
                             , 'meta_type'      : 'Report'
                             , 'description'    : """\
Tabular report"""
                             , 'icon'           : 'reportitem_icon.gif'
                             , 'product'        : 'CMFNauTools'
                             , 'factory'        : 'addReport'
                             , 'immediate_view' : 'report_options_form'
                             , 'actions'        :
                                ( { 'id'            : 'view'
				  , 'name'          : 'View'
                                  , 'action'        : 'report_view'
                                  , 'permissions'   : (
					CMFCorePermissions.View,
                                     )
                                  },
                                  { 'id'            : 'edit'
				  , 'name'          : 'Report options'
                                  , 'action'        : 'report_options_form'
                                  , 'permissions'   : (
					CMFCorePermissions.ModifyPortalContent,
                                     )
                                  },
                                )
                             }
                           ,
                           )

def addReport(self, id, title='', description=''):
    """ Add an HTML document """
    o = Report(id, title, description)

    self._setObject(id, o)


class Report(SimpleNauItem, Document, ManageCMFContent):
    """ Report type """
    meta_type = 'Report'
    security = ClassSecurityInfo()

    manage_options = ( Document.manage_options )

    security = ClassSecurityInfo()

    def __init__(self, id, title='', description=''):
        DefaultDublinCoreImpl.__init__(self)
        self.id = id
        self.title = title
        self.description = description

        self.resp_groups=[]
        self.resp_members=[]
        self.closed=None

        self.propertysheets.manage_addPropertySheet('reportconf', 'reportconf')

    def __call__(self):
        """\
           Invokes the default view.
        """

        view = _getViewFor(self)
   
        if getattr(aq_base(view), 'isDocTemp', 0):
            return apply(view, (self, self.REQUEST))
        else:
            return view()

    security.declareProtected( CMFCorePermissions.View, 'config' )
    def config( self, title, expiration_date, description='', resp_groups=[], resp_members=[], REQUEST=None ):
        """\
           Add new field to the report table
        """
        self.editMetadata( title=title
                         , description=description
                         , expiration_date=expiration_date
                         )

        self.resp_groups=resp_groups
        self.resp_members=resp_members

        if self.resp_members:
            for u_id in self.resp_members:
                 if 'Owner' not in self.get_local_roles_for_userid(u_id):
                     self.manage_setLocalRoles(u_id, ['Reader'])

        if self.resp_groups:
            for g_id in self.resp_groups:
                 self.manage_setLocalGroupRoles(g_id, ['Reader'])
    
        self.reindexObject()

        if REQUEST is not None:
             message = ""
             reportconf = self.propertysheets.get('reportconf')
             r = REQUEST.get

             if r('add_field'):
                fname = r('fname')
                ftype = r('ftype')

                fname = fname.replace(' ', '_')

                if ftype == 'date':
                     default_value = '00:00'
                elif ftype in ['int', 'float']:
                     default_value = '0'
                else:
                     default_value = ''

                if fname:
                    reportconf.manage_addProperty(fname, default_value, ftype)
                    message = "Field added"

                REQUEST[ 'RESPONSE' ].redirect(self.absolute_url() + '/report_options_form'
                                              + '?portal_status_message=' + message)
             elif r('del_fields'):
                ids=[]
                for id in reportconf.propertyIds():
                   if hasattr(REQUEST, id): ids.append(id)

                if ids:
                    reportconf.manage_delProperties(ids)
                    message = "Field removed"

                REQUEST[ 'RESPONSE' ].redirect(self.absolute_url() + '/report_options_form'
                                              + '?portal_status_message=' + message)
             else:
                if not reportconf.propertyIds():
                    message = "You have to specify at least one report field"
                    REQUEST[ 'RESPONSE' ].redirect(self.absolute_url() + '/report_options_form'
                                                  + '?portal_status_message=' + message)
                else:
                    message = "Changes saved"
                    REQUEST[ 'RESPONSE' ].redirect(self.absolute_url()
                                                  + '?portal_status_message=' + message)

    security.declareProtected( CMFCorePermissions.View, 'delReport' )
    def delReport( self, REQUEST ):
        """
           Remove row from the report table
        """
        for key, value in REQUEST.items():
          if key[:11] == 'reportitem_':
              # Get the user id and timestamp from request
              ts = key[-10:]
              uname = key[11:-11]
              # Remove all records containing given timestamp 
              # from the member's propertysheet
              ids=[]
              membersheet = self.propertysheets.get(uname)
              for f_id in membersheet.propertyIds():
                   if f_id[-10:] == ts:
                       ids.append(f_id)

              membersheet.manage_delProperties(ids)

        message = "Report deleted"
        if REQUEST is not None:
            REQUEST[ 'RESPONSE' ].redirect(self.absolute_url()
                                          + '?portal_status_message=' + message
                                          )

    security.declareProtected( CMFCorePermissions.View, 'addReport' )
    def addReport( self, report=None, uname=None, REQUEST=None ):
        """\
           Add row to the report table
        """
        if uname is None:
           membership = getToolByName(self, 'portal_membership')
           uname = membership.getAuthenticatedMember().getUserName()

        if uname not in self.involvedUsers():
           return

        reportconf = self.propertysheets.get('reportconf')
        if report is None and REQUEST is not None:
            report = {}

            for fname in reportconf.propertyIds():
              ftype = reportconf.getPropertyType(fname)
              if ftype == 'date':
                  report[fname] = DateTime('%d/%s/%d %d:%d' % ( REQUEST.get(fname + '_year')
                                          , REQUEST.get(fname + '_month')
                                          , REQUEST.get(fname + '_day')
                                          , REQUEST.get(fname + '_hour')
                                          , REQUEST.get(fname + '_min')
                                          ) 
                                 )
              else:
                  report[fname] = REQUEST.get(fname)

        membersheet = self.propertysheets.get(uname, None)
        if membersheet is None:
            self.propertysheets.manage_addPropertySheet(uname, uname)
            membersheet = self.propertysheets.get(uname)

        reportId = int(self.ZopeTime())

        for fname in report.keys():
           if reportconf.hasProperty(fname):
               ftype = reportconf.getPropertyType(fname)
               membersheet.manage_addProperty("%s_%d" % (fname, reportId), report[fname], ftype)

        message = "Report added"
        REQUEST[ 'RESPONSE' ].redirect(self.absolute_url()
                                      + '?portal_status_message=' + message
                                      )

    security.declareProtected( CMFCorePermissions.View, 'reports' )
    def reports( self, from_date = None, till_date=None ):
        """\
           Return reports
        """
        results = {}
        reportconf = self.propertysheets.get('reportconf')
        for u_id in self.involvedUsers():
            membersheet = self.propertysheets.get(u_id, None)
            if membersheet is not None:
                for propId, value in membersheet.propertyItems():
                     ## Timestamp is a key
                     ts = propId[-10:]
                     if (from_date is None and till_date is None) or \
                        int(ts) >= int(from_date) and int(ts) <= int(till_date):
                         fname = propId[:-11]
                         if not results.get(ts, None): 
                             results[ts] = {}
                             results[ts]['Creator'] = u_id
                             results[ts]['TimeStamp'] = self.ZopeTime(int(ts)) 
     
                         results[ts][fname] = value

        return map( lambda x, r=results: ( r.get(x) ), results.keys()) 

    security.declareProtected( CMFCorePermissions.View, 'reports' )
    def reportCount( self ):
        """\
           Count reports
        """
        results = {}
        reportconf = self.propertysheets.get('reportconf')
        for u_id in self.involvedUsers():
            membersheet = self.propertysheets.get(u_id, None)
            if membersheet is not None:
                for propId, value in membersheet.propertyItems():
                     ## Timestamp is a key
                     ts = propId[-10:]
                     results[ts] = 1

        return len(results.keys())

    security.declareProtected( CMFCorePermissions.View, 'involvedUsers' )
    def involvedUsers( self, status=None ):
        """\
           Involved members list
        """
        members_list = []

        if self.resp_members:
            members_list += self.resp_members

        groups_tool = getToolByName(self, 'portal_groups')
        if self.resp_groups:
          for group in self.resp_groups:
               group_users = groups_tool.group_users(group)
               for user in group_users:
                  if user not in members_list: members_list.append(user)

        return members_list

    security.declareProtected( CMFCorePermissions.View, 'reportSheet' )
    def reportSheet( self ):
        sheet = self.propertysheets.get('reportconf').__of__(aq_base(self))
        return sheet
Globals.InitializeClass(Report)

def install(target):
     """\
        Registering Report class
     """
     Installer = ManageCMFContent()

     Installer.deploy_class( target=target
                      , type_name='Report'
                      , fti=factory_type_information
                      )
