from Globals import InitializeClass
from OFS.SimpleItem import SimpleItem
from AccessControl import ClassSecurityInfo, Permissions
from AccessControl.Role import RoleManager
from Products.CMFCore.utils import getToolByName
from Acquisition import aq_parent, aq_base, aq_inner
from Products.CMFCore import CMFCorePermissions
from string import join, split
from urllib import quote


class SimpleNauItem:
    """\
	A suite of methods generic for all CMFNauTools classes
    """
    security = ClassSecurityInfo()

    def absolute_url(self, relative=0):
        try:
            req = self.REQUEST
        except AttributeError:
            req = {}
        rpp = req.get('VirtualRootPhysicalPath', ('',))
        spp = self.getPhysicalPath()
        i = 0
        for name in rpp[:len(spp)]:
            if spp[i] == name:
                i = i + 1
            else:
                break
        path = map(quote, spp[i:])
        # This is useful for physical path relative to a VirtualRoot
        if relative:
             return join(path, '/')
        else:
             return '/' + join(path, '/')

    def parent_url(self):
        return self.aq_parent.absolute_url()

    def full_size(self):
        """
          Returns size of the object including subobjects
        """
        sum = 0
        
        if hasattr(self.aq_self, 'get_size'):
              sum = sum + self.get_size()

        for obj in self.objectValues():
            if hasattr(obj.aq_self, 'full_size'):
               sum = sum + obj.full_size()
            else:
               sum = sum + obj.get_size()
        
        return sum

    def getEmail(self, user_ids, additional_info = 0):
          """\
             Get user's email addresses from id
          """
          membership = getToolByName(self, 'portal_membership')
          emails = []
          for id in list(user_ids):
                member = membership.getMemberById( id )
                
                if member is not None and member.getProperty('email', None):
                     emails.append( member.getProperty( 'email' ) )

          return emails


    security.declareProtected( Permissions.delete_objects, 'delete_me')
    def delete_me(self, REQUEST=None):
        """
          Delete me!
        """
        self.aq_parent._delObject(self.getId())
        if REQUEST is not None:
            REQUEST[ 'RESPONSE' ].redirect( self.aq_parent.absolute_url() )

    def creatorName(self, u_id=None):
        if u_id is None: u_id = self.Creator()
        membership = getToolByName(self, 'portal_membership')
        member = membership.getMemberById( u_id )

	if member is not None:
                fname = member.getProperty('fname', None)
                mname = member.getProperty('mname', None)
                lname = member.getProperty('lname', None)
                fullname = join(filter(None, [fname, mname, lname]), ' ')
                if not fullname: 
                   fullname = u_id

                return fullname
        return u_id

    def creatorEmail(self):
        membership = getToolByName(self, 'portal_membership')
        member = membership.getMemberById( self.Creator() )

	if member is not None:
		return member.getProperty('email')
	else:
		return None

    #
    # External site interaction methods
    #
    security.declareProtected( Permissions.access_contents_information, 'getSite')
    def getSite(self):
        """
          Return current Site object or None. 
        """
        if hasattr(self, 'site_object'):
           return self.site_object()

        return None
   
    security.declareProtected( Permissions.access_contents_information, 'getOwnerSiteContainer')
    def getOwnerSiteContainer(self):
	"""
	   check that current object included in SiteContainer
	   and return dictionary with site container and path to
	   current object from site container 
	"""
        from Products.CMFNauTools.Site import SiteContainer
	currObj = self
	path = []
	while currObj:
	    if currObj.meta_type == "NauSite": return None
	    path.insert(0, currObj.getId())
	    if isinstance(currObj, SiteContainer): 
		return {'object':currObj, 'path':path}
	    currObj = currObj.aq_parent
	return None

    security.declarePublic('external_url')
    def external_url(self):
        """
          Apply new URL rules
        """
        if not hasattr(self, 'site_storage'):
          return None
        internal_url = self.absolute_url()
        external_url = internal_url.replace( self.site_storage(), "%s/%s/go" % (self.repository_url, self.siteId()) )

        return external_url

    security.declareProtected( Permissions.access_contents_information, 'isHomePage')
    def isHomePage(self):
	"""
	   Check whether the document was set as a 
           site start page
	"""
        site = self.getSite()
        if site is not None:
           home_page = site.getHomePage()
           if home_page:
              return home_page.absolute_url(1) == self.absolute_url(1)

        return None

    #
    # Site presentation interface methods
    #
    def isPublished(self):
        """
           Check whether document is published on the 
           external site or not
        """
        if self.getSite() is None:
           return None

        workflow_tool = getToolByName(self, 'portal_workflow')
        published = workflow_tool.getInfoFor(self, 'review_state') == 'published'

        return published

    def isIndexDocument(self, flag='return flag value'):
        """Set mark on the document, that it must be used
           as index page for topic on external site
        """
        if flag == 'return flag value':
            return getattr(self, 'is_index_document', 0)
        else:
            topic = self.aq_parent
            if flag:
                topic.setDisplayMode(1)
            else:
                topic.setDisplayMode(0)

            self.is_index_document = flag

    def setNewsLockStatus(self, flag=0):
        self.newsLockStatus = flag 

    def getNewsLockStatus(self):
        if hasattr(self, 'newsLockStatus'):
           return self.newsLockStatus
        return None

    security.declareProtected( CMFCorePermissions.ManageProperties, 'setNewsMode')
    def setNewsMode(self, mode=''):
        """
           Set news mode of the document
           mode: primary, secondary or None
        """
        # Retrieve the site object
        site = self.getSite()

        if site is None:
           return None

        # Reset site presentation settings
        site.setOrdinaryDocument(self)

        if mode == "primary":
            site.setPrimaryDocument(self)
        if mode == "secondary":
            site.addSecondaryDocument(self)

        self.newsLockStatus = 0

        return 1
													  
    def getNewsMode(self):
        """
          Get the way document
        """
        site = self.getSite()
        if site is None:
           return None

        if self.absolute_url(1) == site.primary_document:
           mode = 'primary'
        elif self.absolute_url(1) in site.secondary_documents:
           mode = 'secondary'
        else:
           mode = None

        return mode

