"""Site content class.

$Id: Site.py,v 1.16 2002/09/24 12:37:32 ikuleshov Exp $
"""
import Globals
from Globals import InitializeClass, DTMLFile, HTML
from AccessControl import ClassSecurityInfo, getSecurityManager
from Products.CMFCore import CMFCorePermissions
from Products.CMFCore.utils import getToolByName

from Products.CMFDefault.Portal import PortalGenerator
from Products.CMFDefault import cmfdefault_globals

from Products.CMFCore.CMFCorePermissions import View, ManageProperties, ListFolderContents, ModifyPortalContent
from Products.CMFCore.CMFCorePermissions import AddPortalFolders, AddPortalContent, ChangePermissions
from Products.CMFCore.CMFCatalogAware import CMFCatalogAware
from Products.CMFCore import PortalFolder

from ManageCMFContent import ManageCMFContent

from cStringIO import StringIO
from string import join

import HTMLDocument, Heading, SiteImage, NauPublishTool

factory_type_information = ( { 'id'             : 'Site Container'
                             , 'meta_type'      : 'Site Container'
                             , 'description'    : """\
External site container."""
                             , 'icon'           : 'site_icon.gif'
                             , 'product'        : ''
                             , 'factory'        : ''
                             , 'filter_content_types' : 0
                             , 'immediate_view' : ''
                             , 'actions'        :
                                ( { 'id'            : 'view'
                                  , 'name'          : 'View'
                                  , 'action'        : 'folder_contents'
                                  , 'permissions'   : (View,
                                     )
                                  , 'category'      : 'folder'
                                  },
                                  { 'id'            : 'edit'
                                  , 'name'          : 'Edit'
                                  , 'action'        : 'folder_contents'
                                  , 'permissions'   : (View,
                                     )
                                  , 'category'      : 'folder'
                                  },
                                )
                             }
                           ,
                           )

ManagedRoles = ['Editor', 'Reader', 'Writer', 'Author', ]

class SiteContainer( Heading.Heading  ):
    """ Site Container class """
    meta_type = 'Site Container'
    portal_type = 'Site Container'

    ManagedRoles = ManagedRoles

    security = ClassSecurityInfo()
    def __init__(self, id, title=''):
        self.id = id
        self.title = title

        self.primary_document = None        
        self.secondary_documents = []
        self.secondary_max_count = 3
        self.locked_documents = []
        self.locked_max_count = 1						
        
        self.home_page = None
        self.repository_url = None

    security.declareProtected( View, 'installed_plugins')
    def installed_plugins(self):
        """
          Return the list of installed site plugins
        """
        site = self.unrestrictedTraverse( "%s/%s" % (self.repository_url, self.id) )
        return getattr(site, 'plugins', [])

    security.declareProtected( View, 'site_storage')
    def site_storage(self, relative=None):
        """\
           Return a site storage url
        """
        return self.absolute_url(relative) + '/' + self.storage_id

    security.declareProtected( View, 'site_url')
    def site_url(self, relative=None):
        """\
           Return a site url
        """
        return self.absolute_url(relative)

    security.declareProtected( View, 'site_object')
    def site_object(self, relative=None):
        """\
           Return a site object
        """
        return self

    security.declareProtected( View, 'site_object')
    def external_root(self, relative=None):
        """\
           Return a site object
        """
        return '%s/%s' % (self.repository_url, self.getId())

    security.declareProtected( View, 'siteId')
    def siteId(self):
        """\
           Return site id
        """
        return self.getId()

    security.declareProtected( AddPortalFolders, 'manage_addSiteContainer')
    def manage_addSiteContainer(self, id, title='', skin_links=[], storage_id='storage', repository_url=None, plugins=[]):
        """\
           Add a new SiteContainer object with id *id*.
        """
        ob=SiteContainer(id, title)
        self._setObject(id, ob)
        self.reindexObject()
	
        # Create the document storage
        target=ob.__of__(self)
        target.manage_addProduct['CMFNauTools'].manage_addHeading(id=storage_id, )

        storage_url = '/%s/%s' % (target.absolute_url(1), storage_id)
        ob.storage_id = storage_id
        ob.repository_url = repository_url
        repository = self.unrestrictedTraverse( repository_url, None )

        # Create the publisher entry point inside the external CMF Site.
        # It points directly to the document storage.
        gen = SiteGenerator()
        p = gen.create(target, repository, id, title, plugins, 0)
        p.manage_addProduct['CMFNauTools'].manage_addNauPublisher(id='go', internal=storage_url )
        
        # Now create links to the CMF Site folders in the Site Container object.
        # This will enable user to access the contents of those folders.
        # The main (and only?) porpose of this feature is to provide skin
        # editing ability.
        skin_ids = []
        for link in skin_links:
          ob = self.unrestrictedTraverse(p.absolute_url(1) + '/' + link, None)
          if ob is not None:
              link_id = link.split('/')
              link_id = link_id[-1]
              target._setObject(link_id, ob)
              target[link_id].manage_addProperty('ext_link', link, 'text')

              skin_ids.append(link_id)

        if 'custom' in skin_ids:
            defaults = self.unrestrictedTraverse(p.absolute_url(1) + '/portal_skins/site_common', None)
            f = target['custom']
            for id, value in defaults.objectItems():
                if hasattr(value, 'document_src'):
                  file = StringIO()
                  file.write( value.document_src() )
                  file.seek(0)
                  f.manage_addProduct['CMFNauTools'].addDTMLDocument(id=id, file=file)

        if 'images' in skin_ids:        
            images = self.unrestrictedTraverse(p.absolute_url(1) + '/portal_skins/site_images', None)
            f = target['images']
            for id, value in images.objectItems():
               f.manage_addProduct['CMFNauTools'].addSiteImage(id=id, file=value._readFile(1))
               ob = f._getOb( id )
               if hasattr(ob, '_setPortalTypeName'):
                  ob._setPortalTypeName('Site Image')

        # Invoke installer for each plugin
        portal_url = getToolByName(repository, 'portal_url')
        for plugin in plugins:
           installer = portal_url.getPortalObject().plugins[plugin]
           apply(installer.__of__(p), ())

    def manage_beforeDelete(self, item, container):
        """
          Remove the CMF Site bound to the SiteContainer object
        """
        # Invoke the standart unindexing routine
        Heading.Heading.manage_beforeDelete(self, item, container)

        repository = self.unrestrictedTraverse( self.repository_url, None )
        if repository is not None:
            # wipe out CMF Site from it's repository
            repository.manage_delObjects(self.id)

    #
    # Documents presentation management interface
    #
    security.declareProtected( ManageProperties, 'setOrdinaryDocument')
    def setOrdinaryDocument(self, ob):
        """
           Set new primary document of the current SiteContainer
        """
        url = ob.absolute_url(1)
        if url == self.primary_document:
           self.primary_document = None

        if url in self.secondary_documents:
           self.secondary_documents.remove(url)

        self._p_changed = 1

        return

    security.declareProtected( ManageProperties, 'setPrimaryDocument')
    def setPrimaryDocument(self, ob):
        """
           Set new primary document of the current SiteContainer
        """
        self.primary_document = ob.absolute_url(1)

        return

#    security.declareProtected( View, 'getPrimaryDocument')
    security.declarePublic( 'getPrimaryDocument')
    def getPrimaryDocument(self):
        """
           Get the primary site document
           Return None if no document was set
        """
        if self.primary_document:
           result = self.unrestrictedTraverse(self.primary_document, None)
           # document was moved or deleted
           if result is None:
              self.primary_document = None
        else:
           result = None

        return result
	
    security.declareProtected( ManageProperties, 'addSecondaryDocument')
    def addSecondaryDocument(self, ob):
        """
           Add document to the list of secondary documents,
           remove previous news if necessary
        """
        secondary_documents = self.secondary_documents

        if len(secondary_documents) >= self.secondary_max_count:
           secondary_documents.pop(0)

        secondary_documents.append(ob.absolute_url(1))

        self._p_changed = 1

        return

    security.declarePublic( 'getSecondaryDocuments')
    def getSecondaryDocuments(self):
        """
           Add document to the list of secondary documents,
           remove previous news if necessary
        """
        results = []
        for url in self.secondary_documents:
            ob = self.unrestrictedTraverse(url, None)

            # document was moved or deleted
            if ob is None:
               self.secondary_documents.remove(url)
            else:
               results.append(ob)

        return results

    security.declareProtected( ManageProperties, 'lockSecondaryDocument')
    def lockSecondaryDocument(self, ob):
        """
           Lock document in the secondary news box
        """
        length = len(self.lockedDocuments)
        if length >= self.lockedMaxCount:
           docs_with_date = [(document.Date(), document) for document in self.lockedDocuments]
           docs_with_date.sort()
           self.lockedDocuments = [pair[1] for pair in docs_with_date]
           self.lockedDocuments[length - 1].lockStatus = 0
           del self.lockedDocuments[length - 1]

        self.secondaryDocuments.append(ob)
	ob.lockStatus = 1
	
    def setSecondaryMaxCount(self, value=3):
        self.secondary_max_count = value
        return
	    
    def getSecondaryMaxCount(self):
        return self.secondary_max_count
				
    def setLockedMaxCount(self, value=1):
        if value >= self.secondary_max_count:
           value = self.secondary_max_count - 1
        self.locked_max_count = value
							       
    def getLockedMaxCount(self):
        return self.locked_max_count

    security.declareProtected( ModifyPortalContent, 'setHomePage')
    def setHomePage(self, ob):
        """
          Set site start page
        """
        if ob is None:
           self.home_page=None
           return
 
        page_url = ob.absolute_url(1)
        self.home_page = page_url
        return

    def getHomePage(self):
        """
          Return site start page or None
        """
        page_url = self.home_page
        if page_url:
           result = self.unrestrictedTraverse(page_url, None)
        else:
           result = None
        return result

 
Globals.InitializeClass(SiteContainer)

addSiteContainer = SiteContainer.manage_addSiteContainer


default_factories = ( NauPublishTool.factory_type_information
                    + HTMLDocument.factory_type_information
                    + Heading.factory_type_information
                    + SiteImage.factory_type_information
                    )

class SiteGenerator(PortalGenerator):

    def setupTools(self, p):
        """Set up initial tools"""

        addCMFCoreTool = p.manage_addProduct['CMFCore'].manage_addTool
        addCMFCoreTool('CMF Actions Tool', None)
        addCMFCoreTool('CMF Skins Tool', None)
        addCMFCoreTool('CMF Types Tool', None)
        addCMFCoreTool('CMF Undo Tool', None)

        addCMFDefaultTool = p.manage_addProduct['CMFDefault'].manage_addTool
        addCMFDefaultTool('Default URL Tool', None)
        addCMFDefaultTool('Default Properties Tool', None)

    def setupDefaultSkins(self, p):
        ps = getToolByName(p, 'portal_skins')

        cmf_manager = ManageCMFContent()

        cmf_manager.register_view(p, 'site_mandatory')
        cmf_manager.register_view(p, 'site_common')
        cmf_manager.register_view(p, 'site_images')
        cmf_manager.register_view(p, 'generic')

        ps.manage_addProduct['CMFNauTools'].manage_addHeading(id='custom')
        ps.manage_addProduct['CMFNauTools'].manage_addHeading(id='images')
        default_skins = ['site_mandatory', 'custom', 'images', 'site_common', 'site_images', 'generic']

        try:
          import Products.Tracker
          cmf_manager.register_view(p, 'tracker')
          default_skins.append('tracker')
        except:
          pass

        ps.addSkinSelection('Site',
            join(default_skins, ','),
            make_default=1)
        p.setupCurrentSkin()

    def setupPermissions(self, p):
        # Set up some suggested role to permission mappings.
        mp = p.manage_permission

        mp('List folder contents', ['Anonymous',], 1)
        mp('View', ['Anonymous',], 1)

    def setup(self, p, create_userfolder):
        self.setupTools(p)
        if int(create_userfolder) != 0:
            self.setupUserFolder(p)
        self.setupRoles(p)
        self.setupPermissions(p)
        self.setupDefaultSkins(p)

        #   SkinnedFolders are only for customization;
        #   they aren't a default type.
        default_types = tuple( filter( lambda x: x['id'] != 'Skinned Folder'
                                     , default_factories ) )
        self.setupTypes(p, default_types )

        self.setupTypes(p, PortalFolder.factory_type_information)
        self.setupMimetypes(p)

    def create(self, container, repository, id, title, plugins, create_userfolder):
        id = str(id)
        portal = self.klass(id=id)
        repository._setObject(id, portal)
        # Return the fully wrapped object.
        p = repository.this()._getOb(id)
        self.setup(p, create_userfolder)

        portal.setTitle(title)

        p.internal_url = join(container.getPhysicalPath(), '/')
        p.plugins = plugins

        return p
