"""Task Item content class.

$Id: TaskItem.py,v 1.8 2002/09/10 12:00:23 ikuleshov Exp $
"""
import re, string, os.path
import Globals

from Globals import DTMLFile, Persistent, PersistentMapping
from Acquisition import aq_base
from AccessControl import ClassSecurityInfo, Permissions
from DateTime import DateTime
from string import join

from HTMLDocument import HTMLDocument
from HTMLCleanup import HTMLCleaner

from Products.CMFDefault.DiscussionItem import DiscussionItem, DiscussionItemContainer
from Products.CMFCore.utils import getToolByName, _getViewFor
from Products.CMFCore.ActionInformation import ActionInformation
from Products.CMFCore.Expression import Expression
from Products.CMFCore import CMFCorePermissions


factory_type_information = ( { 'id'             : 'Task Item'
                             , 'meta_type'      : 'Task Item'
                             , 'description'    : """\
Followup task item"""
                             , 'icon'           : 'taskitem_icon.gif'
                             , 'product'        : '' # leave blank to suppress
                             , 'factory'        : ''
                             , 'immediate_view' : ''
                             , 'actions'        :
                                ( { 'id'            : 'view'
                                  , 'name'          : 'View task'
                                  , 'action'        : 'task_view'
                                  , 'permissions'   : (
                                      CMFCorePermissions.View, )
                                  }
                                ,
                                )
                             }
                           ,
                           )


class TaskItem(DiscussionItem):
    """ Task item """
    meta_type = 'Task Item'

    _actions = [ActionInformation(id='report'
                            , title='Report wizard'
                            , action=Expression(
            text='string: ${object_url}/report_wizard_form')
                            , category='object'
                            , permissions=(CMFCorePermissions.ModifyPortalContent,)
                            , visible=1
                            ),
                ActionInformation(id='commit'
                            , title='Commit task'
                            , action=Expression(
            text='string: ${object_url}/task_commit_form')
                            , condition=Expression(
            text='python: member.id in object.involvedUsersFor(object)')
                            , category='object'
                            , permissions=(CMFCorePermissions.View,)
                            , visible=1
                            ),
                ActionInformation(id='reject'
                            , title='Reject task'
                            , action=Expression(
            text='string: ${object_url}/task_reject_form')
                            , condition=Expression(
            text='python: member.id in object.involvedUsersFor(object)')
                            , category='object'
                            , permissions=(CMFCorePermissions.View,)
                            , visible=1
                            ),
                ActionInformation(id='subtask'
                            , title='Define a subtask'
                            , action=Expression(
            text='string: ${object_url}/task_add_form')
                            , condition=Expression(
            text='python: member.id in object.involvedUsersFor(object) or member.id == object.Creator()')
                            , category='object'
                            , permissions=(CMFCorePermissions.View,)
                            , visible=1
                            ),
                ActionInformation(id='close'
                            , title='Close task'
                            , action=Expression(
            text='string: ${object_url}/task_close_form')
                            , condition=Expression(
            text='python: member.id == object.Creator()')
                            , category='object'
                            , permissions=(CMFCorePermissions.View,)
                            , visible=1
                            ),
                ActionInformation(id='review'
                            , title='Review'
                            , action=Expression(
            text='string: ${object_url}/task_review_form')
                            , condition=Expression(
            text='python: member.id == object.Inspector()')
                            , category='object'
                            , permissions=(CMFCorePermissions.View,)
                            , visible=1
                            ),]
 
    security = ClassSecurityInfo()

    def __init__(self, id, title='', description=''):
        DiscussionItem.__init__( self, id, title, description)
        self.resp_groups=[]
        self.resp_members=[]
        self.reviewed=None
        self.closed=None

    def __call__(self):
        """
           Invokes the default view.
        """
        view = _getViewFor(self)
   
        if getattr(aq_base(view), 'isDocTemp', 0):
            return apply(view, (self, self.REQUEST))
        else:
            return view()

    security.declarePrivate('listActions')
    def listActions(self, info=None):
        """
          Return actions provided by tool.
        """
        _actions = self._actions

        return _actions

    security.declareProtected( CMFCorePermissions.View, 'isResponsible' )
    def isResponsible( self ):
        membership = getToolByName(self, 'portal_membership')
        uname = membership.getAuthenticatedMember().getUserName()

        return uname in self.involvedUsersFor(self)

    security.declareProtected( CMFCorePermissions.View, 'comment' )
    def comment( self, uname=None, report=None ):
        """
           Retrieve the comment of the given member
           If 'report' is not None, returns the text that was included 
           to the overall report, not the original user comment.
        """
        if uname is None:
           membership = getToolByName(self, 'portal_membership')
           uname = membership.getAuthenticatedMember().getUserName()

        if report is not None:
           report_prop = 'r_' + uname
           if hasattr(self, report_prop):
              text = self.getProperty(report_prop)
              return text
           
        comment_prop = 'c_' + uname
        if hasattr(self, comment_prop):
           text = self.getProperty(comment_prop)
           return text
        return ''
      

    security.declareProtected( CMFCorePermissions.View, 'review' )
    def review( self, comment=None, REQUEST=None ):
        """
           Inspector reviewes task
        """
        membership = getToolByName(self, 'portal_membership')
        uname = membership.getAuthenticatedMember().getUserName()
        if uname != self.Inspector():
           return

        self.reviewed = 1
        self._taskStatus( '_inspector', comment, 1, REQUEST)

    security.declareProtected( CMFCorePermissions.View, 'commit' )
    def commit( self, comment=None, REQUEST=None ):
        """
           Commit to a task
        """
        membership = getToolByName(self, 'portal_membership')
        uname = membership.getAuthenticatedMember().getUserName()
        if uname not in self.involvedUsersFor(self):
           return

        self._taskStatus(uname, comment, 1, REQUEST)

    security.declareProtected( CMFCorePermissions.View, 'reject' )
    def reject( self, comment=None, REQUEST=None ):
        """
           Reject task
        """
        membership = getToolByName(self, 'portal_membership')
        uname = membership.getAuthenticatedMember().getUserName()
        if uname not in self.involvedUsersFor(self):
           return

        self._taskStatus(uname, comment, 2, REQUEST)

    security.declareProtected( CMFCorePermissions.View, 'close' )
    def close( self, comment=None, REQUEST=None ):
        """
           Close a task thread
        """
        membership = getToolByName(self, 'portal_membership')
        uname = membership.getAuthenticatedMember().getUserName()
        if uname != self.creator: 
            return

        self.closed = 1

        self._taskStatus( '_closer', comment, 1)

        message = "Task closed"
        if REQUEST is not None:
            REQUEST[ 'RESPONSE' ].redirect( self.absolute_url()
                          + '?portal_status_message=' + message
                          )           

    security.declarePrivate('_taskStatus')
    def _taskStatus( self, uname, comment=None, status=0, REQUEST=None ):
        """
          Add a user comment to the property sheet 
          with the status given
        """
        comment_prop = 'c_' + uname
        status_prop = 's_' + uname

        # create fields if they does not exist
        if not hasattr(self, comment_prop):
           self.manage_addProperty(comment_prop, '', 'text')
        if not hasattr(self, status_prop):
           self.manage_addProperty(status_prop, 0, 'int')

        apply(self.manage_changeProperties, (), {comment_prop: comment, status_prop: status})

        message = "Task status changed"
        if REQUEST is not None:
            REQUEST[ 'RESPONSE' ].redirect( self.absolute_url()
                          + '?portal_status_message=' + message
                          )           

    security.declareProtected( Permissions.delete_objects, 'delete_me')
    def delete_me(self, REQUEST=None):
        """
          Delete me!
        """
        message = "Task removed"
        self.followup.deleteReply(self.id)

        if REQUEST is not None:
            REQUEST[ 'RESPONSE' ].redirect( self.aq_parent.aq_parent.absolute_url() 
                        + '/document_follow_up_form?portal_status_message=' + message
                        )

    #
    #   ReportResponse interface
    #
    security.declareProtected( CMFCorePermissions.View, 'inReplyTo' )
    def inReplyTo( self, REQUEST=None ):
        """
            Two cases obtain:

              - We are a "top-level" reply to a non-DiscussionItem piece
                of content;  in this case, our 'in_reply_to' field will
                be None.
            
              - We are a nested reply;  in this case, our 'in_reply_to'
                field will be the ID of the parent DiscussionItem.
        """
        followup = self.getTasksFor( self )
        return followup._getReplyParent( self.in_reply_to )

    security.declareProtected( CMFCorePermissions.View, 'Inspector' )
    def Inspector( self ):
        """
            Get an inspector assigned to this (or parent) task
        """
        obj = self
        while(hasattr(obj, 'inReplyTo')):
          if hasattr(obj, 'inspector'):
             return obj.inspector
          obj = obj.inReplyTo()

    security.declareProtected( CMFCorePermissions.View, 'isClosed' )
    def isClosed( self ):
        """
            Recursively check whether this task is closed
        """
        obj = self
        while(hasattr(obj, 'inReplyTo')):
          if hasattr(obj, 'closed') and obj.closed is not None:
             return 1
          obj = obj.inReplyTo()

        return None

Globals.InitializeClass(TaskItem)

class TasksContainer(DiscussionItemContainer):
    """ Tasks container """
    security = ClassSecurityInfo()

    def __init__(self):
        self.id = 'followup'
        self._container = PersistentMapping()

    #
    #   OFS.ObjectManager query interface.
    #
    security.declareProtected( CMFCorePermissions.AccessContentsInformation
                             , 'objectIds' )
    def objectIds( self, spec=None ):
        """
            Return a list of the ids of our Tasks.
        """
        if spec and spec is not Task.meta_type:
            return []
        return self._container.keys()

    security.declareProtected( CMFCorePermissions.View
                             , 'assignedTasks' )
    def assignedTasks(self, uname=None, spec=None):
        """
           Return a list of all tasks assigned to the 
           current user
        """
        if uname is None:
           membership = getToolByName(self, 'portal_membership')
           uname = membership.getAuthenticatedMember().getUserName()

        r=[]
        a=r.append
        g=self._container.get
        for id in self.objectIds(spec):
           task = g( id ).__of__( self )
           if uname in self.involvedUsersFor(task): a( (id, task ) )
        return r

    security.declareProtected( CMFCorePermissions.View, 'saveAs' )
    def saveAs(self, title, description, report_generator, REQUEST):
        """
           Create an HTMLDocument containing a report 
           previously generated with ReportWizard
        """
        id = str( int(DateTime().timeTime()) )
        html_text = report_generator(self, REQUEST)
        if REQUEST.get('no_css', None) is not None:
            html_text = HTMLCleaner(html_text, None, 2, '', 'SCRIPT STYLE')
        
        o = HTMLDocument(id, title, description, 'html', html_text)

        # Find the folder where to place the overall report
        folder = self
        while(folder.meta_type != 'Heading'):
           folder = folder.aq_parent

        folder._setObject(id, o)
        r = getattr(folder, id)

        return r.absolute_url()

    #
    #   Follow-up control interface
    #
    security.declareProtected( CMFCorePermissions.View, 'createTask' )
    def createTask( self, title, expires, description=None, 
                      inspector=None, Creator=None, resp_groups=None, resp_members=None ):
        """
            Create a task in the Tasks Container
        """
        container = self._container

        # Generate a unique task id
        id = int(DateTime().timeTime())
        while self._container.get( str(id), None ) is not None:
            id = id + 1
        id = str( id )

        item = TaskItem(  id=id
                       ,  title=title
                       ,  description=description
                       )

        item.setExpirationDate(expires)

        # Allow acquisition from top-level tasks
        if inspector:
            item.manage_addProperty('inspector', inspector, 'text')

        item.resp_groups=resp_groups
        item.resp_members=resp_members

        if Creator:
            item.creator = Creator

        item.__of__( self ).indexObject()

        item.setReplyTo( self._getDiscussable() )
 
        self._container[ id ] = item

        base = self.aq_parent

        member_list = []           
        if resp_members:
            member_list += resp_members
        if inspector:
            member_list.append(inspector)

        for u_id in self.involvedUsersFor(item, all=1):
             # Notify the involved users via email
             try:
               mail_text = self.task_mail_template( self
                                                  , self.REQUEST
                                                  , member=u_id
                                                  , task_title=title
                                                  , task_url=self.getReply(id).absolute_url()
                                                  )
               host = self.MailHost
               host.send( mail_text )
             except:
               pass

             # Grant access to the involved users
             if 'Owner' not in base.get_local_roles_for_userid(u_id):
                 base.manage_setLocalRoles(u_id, ['Reader'])

        # Grant access to the involved groups
        if resp_groups:
          for g_id in resp_groups:
             self.manage_setLocalGroupRoles(g_id, ['Reader']) 

        base.reindexObject()

        return id

    security.declareProtected( CMFCorePermissions.View, 'Inspector' )
    def Inspector( self ):
        """
           Stub for the top-level task
        """

        return None

    security.declareProtected( CMFCorePermissions.View, 'involvedUsersFor' )
    def involvedUsersFor( self, task, status=None, all=None ):
        """
           Return the list of all members involved to 
           the task given

           Usage:
              task - task to work with

              status = 1 - retrieve committed users list
              status = 2 - retrieve rejected users list
              status = -1 - retrieve pending users list

              all = 1 - get the list of all members involved
                        including task inspector
        """
        members_list = []
        if task.resp_members:
            members_list += task.resp_members
        try:
          inspector = task.Inspector()
          if inspector not in members_list and all is not None: members_list.append(inspector)
        except:
          pass

        # Create a plain user list from the involved groups enumeration
        groups_tool = getToolByName(self, 'portal_groups')
        if task.resp_groups:
          for group in task.resp_groups:
               group_users = groups_tool.group_users(group)
               for user in group_users:
                  if user not in members_list: members_list.append(user)

        # filter out unwanted usernames
        if status > 0: 
            members_list = filter(lambda x, task=task, status=status: task.getProperty('s_' + x, None) == status, members_list)
        elif status == -1: 
            members_list = filter(lambda x, task=task: not task.getProperty('s_' + x, None), members_list)

        return members_list

    security.declareProtected( CMFCorePermissions.View, 'hasReplies' )
    def hasReplies( self, content_obj ):
        """
            Test to see if there are any subtasks
        """
        outer = self._getDiscussable( outer=1 )
        if content_obj == outer: 
            return not not len( self._container )
        else:
            return not not len( content_obj.followup._getReplyResults() ) 

    security.declareProtected( CMFCorePermissions.View, 'getRepliesIds' )
    def getRepliesIds( self ):
        """
            Return a sequence of the DiscussionResponse objects ids which are
            associated with this Discussable
        """
        result_ids = self._getReplyResults()
        for id in result_ids:
           reply = self.getReply( id )
           sub_ids = self.__of__(reply)._getReplyResults()
           if sub_ids:
               result_ids += sub_ids
        return result_ids

    security.declarePrivate('_repcount')
    def _repcount( self, replies ):
        """  
           Counts the total number of replies by recursing thru 
           the various levels
        """
        count = 0

        for reply in replies:
            count = count + 1

            #if there is at least one reply to this reply
            replies = reply.followup.getReplies()
            if replies:
                count = count + self._repcount( replies )

        return count

Globals.InitializeClass(TasksContainer)
