import Globals
from Globals import InitializeClass, DTMLFile
from AccessControl import ClassSecurityInfo
from OFS.PropertyManager import PropertyManager
from OFS.PropertySheets import PropertySheet

from Products.CMFDefault.DublinCore import DefaultDublinCoreImpl
from Products.CMFCore.PortalContent import PortalContent
from Products.CMFCore import CMFCorePermissions
from Products.CMFNauTools.ManageCMFContent import ManageCMFContent


factory_type_information = ( { 'id'             : 'Vote'
                             , 'meta_type'      : 'Vote'
                             , 'description'    : """\
Polls manager"""
                             , 'icon'           : 'vote_icon.gif'
                             , 'product'        : 'CMFNauTools'
                             , 'factory'        : 'addVote'
                             , 'immediate_view' : 'vote_edit_form'
                             , 'allow_discussion' : 1
                             , 'actions'        :
                                ( { 'id'            : 'view'
                                  , 'name'          : 'View'
                                  , 'action'        : 'vote_view'
                                  , 'permissions'   : (
                                      CMFCorePermissions.View, )
                                  }
                                , { 'id'            : 'results'
                                  , 'name'          : 'Results'
                                  , 'action'        : 'vote_results_view'
                                  , 'permissions'   : (
                                      CMFCorePermissions.View, )
                                  }
                                , { 'id'            : 'edit'
                                  , 'name'          : 'Edit'
                                  , 'action'        : 'vote_edit_form'
                                  , 'permissions'   : (
                                      CMFCorePermissions.ModifyPortalContent, )
                                  }
                                , { 'id'            : 'metadata'
				  , 'name'          : 'Metadata'
                                  , 'action'        : 'metadata_edit_form'
                                  , 'permissions'   : (
                                      CMFCorePermissions.ModifyPortalContent, )
                                  }
                                )
                             }
                           ,
                           )


def addVote( self, id, title=''):
    """
        Add a Vote Item
    """
    o=Vote( id=id, title=title )
    self._setObject(id, o)


class Vote(PortalContent, DefaultDublinCoreImpl, PropertyManager, ManageCMFContent):
    """ 
       Portal vote 
    """
    meta_type = 'Vote'

    effective_date = expiration_date = None
    _isDiscussable = 1
    __implements__ = ( PortalContent.__implements__
                     , DefaultDublinCoreImpl.__implements__
                     )

    manage_options = (
           {'label': 'Edit', 'action': 'vote_edit_form'},
           {'label': 'View', 'action': 'vote_view'},
           ) + PropertyManager.manage_options \


    security = ClassSecurityInfo()

    def __init__( self
                , id
                , title = ''
                , question = ''
                , choices = ''
                , totalvotes = 0
                ):
        DefaultDublinCoreImpl.__init__(self)
        self.id = id
        self.title = title

        # Sheet to store voting settings
        voteconf = self.propertysheets.get('default')
        voteconf.manage_addProperty('Question', question, 'string')
        voteconf.manage_addProperty('Choices', choices, 'lines')
        voteconf.manage_addProperty('TotalVotes', totalvotes, 'int')

    security.declareProtected(CMFCorePermissions.ModifyPortalContent, 'manage_changeVotingSettings')
    def manage_changeVotingSettings(self, Question, Choices, REQUEST):
        """
           Edit voting settings.
        """
        self.manage_changeProperties(REQUEST)

        voteconf = self.propertysheets.get('default')
        voteconf.manage_changeProperties({ 'Question': Question , 'Choices': Choices })
        self.reindexObject()

        REQUEST['RESPONSE'].redirect(self.absolute_url())

    security.declareProtected( CMFCorePermissions.View, 'processVote' )
    def processVote(self, REQUEST=None):
        """
           Process user vote
        """
        if not REQUEST.has_key('responses'):
            return

        voteconf = self.propertysheets.get('default')

        choice = REQUEST['responses']
        curVotes = self.getProperty(choice, 0) + 1
        totVotes = voteconf.getProperty('TotalVotes') + 1


        if not voteconf.hasProperty( choice ):
            voteconf.manage_addProperty( choice, 0, 'int' )

        voteconf.manage_changeProperties({ choice: curVotes })
        voteconf.manage_changeProperties({ 'TotalVotes': totVotes })
 
        REQUEST.RESPONSE.setCookie('voted4' + self.id, '1', path='/', expires='Wed, 19 Feb 202014:28:00 GMT')
        REQUEST.RESPONSE.redirect(REQUEST['origUrl']+'?hasVoted=true')

    security.declareProtected( CMFCorePermissions.View, 'getVoteStr' )
    def getVoteStr(self, votes):
        """
           Convert votes to percent
        """

        if self.TotalVotes:
            return "%.2f" % (100 * votes / self.TotalVotes)
        else:
            return "%s" % (votes)

    security.declareProtected(CMFCorePermissions.ModifyPortalContent,
                                      'manage_edit')
    manage_edit = DTMLFile('vote_edit_form', globals() )

Globals.InitializeClass(Vote)

def install(target):
     """\
        Registering Vote class
     """
     Installer = ManageCMFContent()

     Installer.deploy_class( target=target
                      , type_name='Vote'
                      , fti=factory_type_information
                      )
