import base64

from cStringIO import StringIO
from string import split, rfind, strip, join

from ZEO import ClientStorage
from ZODB import DB
from AccessControl import getSecurityManager


class ZEOSyncer:
    server = None
    client = None

    def __init__(self, client, mount_point):
        """\
           Let us connect to ZEO server
        """
        storage = ClientStorage.ClientStorage( mount_point )
        db = DB( storage )
        conn = db.open()
        root = conn.root()
        self.server = root['Application']

        self.client = client

        return

    def transfer(self, obj, path, folder_classes = None):
        """\
           Create the object copy on remote server
        """

        # export the object
        data = StringIO()
        obj._p_jar.exportFile(obj._p_oid, data)
                                             
        # create a fake file
        file = StringIO()
        file.write( data.getvalue() )
        file.seek(0)

        folder = self.server.restrictedTraverse( path, None )
        if folder is None and folder_classes is not None:
           folder = self.makepath( path, folder_classes )
        
        # now import
        new = folder._p_jar.importFile(file)

        id = new.getId()
        if id in folder.objectIds():
           folder.manage_delObjects([id,])

        folder._setObject(new.getId(), new)

        return

    def makepath(self, s_path, folder_classes):
        """\
           Create the full path to the object
        """
        path = s_path.split('/')
        folder = None

        for i in range(len(path) + 1):
            folder = self.server.restrictedTraverse( join(path[:i], '/'), None )
            if folder is None:
               # Get the folder properties
               local_folder = self.client.restrictedTraverse( join(path[:i], '/') )
               id = local_folder.id
               title = local_folder.title
               meta_type = local_folder.meta_type

               # Create the folder remotely using the corresponding class
               folder_class = folder_classes[ meta_type ]
               parent = self.server.restrictedTraverse( join(path[:i - 1], '/'), None )
               parent._setObject(id, folder_class(id, title))

        folder = self.server.restrictedTraverse( s_path, None )

        return folder

    def compare(self, local_path, remote_path):
        """\
           Compare objects
        """
        remote_obj = self.server.restrictedTraverse( remote_path, None )
        if remote_obj is None: 
           return 'missing'

        local_obj = self.client.restrictedTraverse( local_path, None )
        if local_obj is None:
           return 'extra'


        remote_lmt = str(remote_obj.bobobase_modification_time().toZone('GMT'))
        local_lmt = str(local_obj.bobobase_modification_time().toZone('GMT'))

        if local_lmt > remote_lmt:
           return 'out of date'

        return 'ok'
