# $Id: ForestTagTests.py,v 1.3 2002/08/22 06:13:05 ikuleshov Exp $
# $Log: ForestTagTests.py,v $
# Revision 1.3  2002/08/22 06:13:05  ikuleshov
# works anywhere
#
# Revision 1.2  2002/08/21 11:02:14  ikuleshov
# Zope.debug context breaking fixed (Zope.debug more not used)
#
# Revision 1.1  2002/08/20 10:31:04  and
# Creating new test
#

"""Class for testing ForestTag
1. Creating folders tree - deep 3 width 3
2. Every folder with name ended on '0' is making invisible by resetting 'View' permission
3. Checking that invisible folders are hidden on test page, generated by Forest tag
and visible subfolders in invisible folders are "flow up" in hierarchy.
"""

from ZopeTest import ZopeTestCase

class ForestTagTests(ZopeTestCase):
	""" Class for testing ForestTag \
	
	"""
	include = 3
	user = 'tester'
	password = '123'

	def	setUp(self):
		get_transaction().begin()
                self.app.docs.storage.setLocalRoles('Anonymous User', ['Manager', 'Owner'])

		return
	
	def tearDown(self):
		""" Clearing all garbage (creating in test process) """
		get_transaction().abort()
		return

	def	testConfig(self):
		""" Perfoming test """
		from Products.CMFNauTools.ForestTag	import Forest
		from Products.CMFNauTools.Heading import Heading
		from Products.CMFNauTools.Report import	Report

		#creating test folders tree
		id = 'forest'
		cont = self.app.docs.storage
		if hasattr(cont,id):
			cont.manage_delObjects(id)
		cont._setObject(id,	Heading(id))
		cont = self.app.docs.storage[id]
		cont.setLocalRoles(self.user, ['Manager', 'Owner'])
		self.CreateSubForest(cont, 'folder', self.include)
		cont.reindexSubObjects()
		
#	expr="portal_url.getPortalObject().storage.forest"	\
#================================================================================
		text = self.RunTestScript(cont, ' \
<dtml-forest id="/treestate" \
	expr=forest	\
	content_filter="{ \'Type\':	[\'Heading\', \'Site Container\'] }" \
	secured_tree="1" \
	sort=title_or_id \
> \
&dtml-absolute_url; <br> \
</dtml-forest> \
')

		#print text
		#print 'Cookie "tree-sforest": ', self.GetCookie(text, 'tree-sforest')
		
		#This line don't work as need !!!
		#self.app.REQUEST.RESPONSE.setCookie('tree-sforest', self.GetCookie(text, 'tree-sforest'), path='/', expires='Wed, 19 Feb 2020 14:28:00 GMT')
		
		#must contain this URLs
		assert(text.find('/forest/folder_0/folder_0_0/folder_0_0_1') != -1)
		assert(text.find('/forest/folder_0/folder_0_0/folder_0_0_2') != -1)
		assert(text.find('/forest/folder_0/folder_0_1') != -1)
		assert(text.find('/forest/folder_0/folder_0_2') != -1)
		assert(text.find('/forest/folder_1') != -1)
		assert(text.find('/forest/folder_2') != -1)

		#must not contain this URLs
		assert(text.find('/forest/folder_0/folder_0 ') == -1)
		assert(text.find('/forest/folder_0/folder_0_0 ') == -1)
		assert(text.find('/forest/folder_0/folder_1_0 ') == -1)
		assert(text.find('/forest/folder_0/folder_1_1 ') == -1)
		assert(text.find('/forest/folder_0/folder_1_2 ') == -1)
		assert(text.find('/forest/folder_0/folder_2_1_0 ') == -1)
		assert(text.find('/forest/folder_0/folder_2_1_1 ') == -1)

		url = self.FindExpandFolderURL(text, '/forest/folder_0/folder_0_1')
		# print 'ActivateURL', url
		assert(len(url) != 0)
		
		#Now we can open this url (before MUST BE set cookies)
		#print self.MyZopeDebug(url)
		
#================================================================================
		return

	def	CreateSubForest(self, cont,	folder,	num):
		""" Creating test folders tree """
		from Products.CMFNauTools.Heading import Heading
		
		if num == 0	: return
		for	i in range(self.include):
			id=	folder + '_' + str(i)
			cont._setObject(id,	Heading(id))
			#set access right at folders
			cont[id].manage_permission('View', ['Manager'], 0)
			cont[id].manage_permission('List folder contents', ['Manager'], 0)
			if (i == 0):
				#remove access right at folders
				cont[id].manage_permission('View', [], 0)
				cont[id].manage_permission('List folder contents', [], 0)
			self.CreateSubForest(cont[id], id, num-1)

	def	RunTestScript(self,	cont, script, id='script'):
		""" Executing test DTMLMethod, generating test page using Forest Tag """
		from OFS.DTMLMethod	import DTMLMethod

		if hasattr(cont,id):
			cont.manage_delObjects(id)
		cont._setObject(id,	DTMLMethod(id))
		cont[id].manage_edit(script,'Title')
		#return str(Zope.debug('/docs/storage/forest/script', u=self.user+':'+self.password, s='silent'))
		return self.MyZopeDebug('/docs/storage/forest/script')
		
	def FindExpandFolderURL(self, text, folder):
		""" Finding URL for expanding tree node """
		npos = 0
		while 1:
			tmps = 'A HREF="'
			npos = text.find(tmps, npos)
			if npos == -1:
				break
			npos += len(tmps)
			nendurl = text.find('"', npos)
			assert (nendurl != -1)
			tmps = '/docs/storage'
			nFolder = text.find(tmps, nendurl)
			assert (nFolder != -1)
			nFolder += len(tmps)
			folder_ = folder+' '
			#print '<', folder, '> == <', text[nFolder:nFolder+len(folder_)], '>'
			if folder_ == text[nFolder:nFolder+len(folder_)]:
				return '/docs/storage' + folder + '/' + text[npos:nendurl]
		return ''
		
	def MyZopeDebug(self, url):
		""" Zope.debug wrapper """
                obj = self.app.unrestrictedTraverse(url)
                REQUEST = self.app.REQUEST
                REQUEST['AUTHENTICATED_USER'] = self.user

                return obj(self.app.docs.storage, REQUEST)
                
                """		import ZODB
		import Zope
		import sys
		import cStringIO
		Zope.debug(url, u=self.user+':'+self.password)
		return
		tmp = sys.stdout
		sys.stdout= cStringIO.StringIO()
		Zope.debug(url, u=self.user+':'+self.password)
		result = sys.stdout.getvalue()
		sys.stdout = tmp
		return result
                """
		
	def GetCookie(self, sResponse, sNameCookie):
		""" Getting cookie from sResponse """
		#Set-Cookie: tree-sforest="eJzTiFZ3hIJwC19b9VgdTQAu3wSl"; Path=/
		nBegin = 0
		while 1:
			tmps = 'Set-Cookie: '
			nBegin = sResponse.find(tmps, nBegin)
			if nBegin == -1:
				break
			nBegin += len(tmps)
			tmps = '="'
			nEnd = sResponse.find(tmps, nBegin)
			assert (nEnd != -1)
			sName = sResponse[nBegin:nEnd]
			#print 'sName=', sName
			if sName == sNameCookie:
				nBegin1 = nEnd+len(tmps)
				tmps = '"'
				nEnd1 = sResponse.find(tmps, nBegin1)
				assert (nEnd1 != -1)
				sCookie = sResponse[nBegin1:nEnd1]
				#print 'sCookie=', sCookie
				return sCookie
				nEnd1 += len(tmps)
		return ''
			