#$Id: HTMLDocumentTests.py,v 1.12 2002/08/22 12:24:24 kfirsov Exp $
from ZopeTest import ZopeTestCase
import re

class Dummy:
	filename = None
	
	def __init__(self, fn='just/test.txt'):
	        self.filename=fn

	def seek(t, a=None, b=None): pass
	def write(t): pass
	def close(): pass
	def fileno(): raise 'this sux'
	def flush(): pass
	def read(a=None, b=None): return ''
	def readline(): return ''
	def readlines(): return []
	def tell(t=None): return 0
	def truncate(i): pass
	def writelines(i): pass


class HTMLDocumentTestCase(ZopeTestCase):

	def setUp(self):
	        from Products.CMFNauTools import HTMLDocument
	        self.app.docs.storage.t = HTMLDocument.HTMLDocument('t', 
	        	title='title', description='desc', text_format='html', text='')
                
	def tearDown(self):
		self.app.docs.storage.manage_delObjects('t')
		
  
	def testAddHTMLDocument(self):
		from Products.CMFNauTools import HTMLDocument
		import random
		id = 'test_add_html'
		while getattr(self.app.docs.storage, id, None):
			id = 'test_add_html_' + str( random.randrange(1000000) )
		self.app.docs.storage.manage_addProduct['CMFNauTools'].addHTMLDocument(id, 
			title='title', description='decs', text_format='html', text='')
		t = getattr(self.app.docs.storage, id)
		self.assert_(t, 'AddHtmlDocument Error' )
		self.app.docs.storage.manage_delObjects(id)


	def testCookId(self):
		from OFS import Image
		from cStringIO import StringIO
		t = self.app.docs.storage.t 

	        fle = StringIO()
        	fle.write( 'some data' )
        	fle.seek(0)
		
		self.app.docs.storage.fle = Image.File('fle', 'title', fle)
		self.app.docs.storage.fle.filename="\\docs\\storage\\fle"
		
		self.assertEqual( t.cookId( self.app.docs.storage.fle ),  'fle', 'slashes doesn''t trimmed correctly ')
		self.app.docs.storage.fle.filename="/docs/storage/fle"
		self.assertEqual( t.cookId( self.app.docs.storage.fle ),  'fle', 'slashes doesn''t trimmed correctly ')
		fle.close()
		self.app.docs.storage.manage_delObjects('fle')
		


	def testCleanup(self):
		t = self.app.docs.storage.t
		old_text = '<SCRIPT>body.onload="" </SCRIPT>&nbsp;&nbsp;\
			<STYLE>some stylesheet </STYLE> <font color="#121212" size="12">text</FONT>'
		new_text = t.cleanup(old_text)
		if new_text.find('<SCRIPT>')!= -1 :
			self.fail('The "SCRIPT" tag still in text')

	def check_insert(self, where, what, class_names, filename):
		#check add
		where.addFile( what )
		if filename not in where.objectIds():
       			self.fail('addFile did not attach file: '+what.filename)
       		#check classtype
       		ok = 0
       		for class_name in class_names:
       			if str(getattr(where, filename).__class__).find(' '+class_name+' ') != -1:
				ok = 1
				break
		if not ok:
			self.fail('Wrong class of created file: ' + str(getattr(where, filename).__class__) )
		
	def check_delete(self, where, what, filename, html):
       		trimmed_html = where.deleteFile( filename, html)
       		trimmed_html = re.sub('\s', '', trimmed_html)
       		self.assertEqual( trimmed_html, '<body></body>', 'Link to deleted attach was not removed')
       		if filename in where.objectIds() :
       			self.fail('deleteFile did not remove file: ' + filename)

	def test_AddFile_DelFile_PasteFile_ObjectIds_ObjectValues_ObjectItems(self):

		t = self.app.docs.storage.t

	        fle1 = Dummy('just/test.txt')
	        #just add and remove simple 'file'
	        self.check_insert(t, fle1, ['OFS.Image.File'], 'test.txt')
	        #check pasteFile here
	        ht1=t.pasteFile('test.txt', '<body></body>')
	        ht11='<body><a href="t/test.txt">test</a></body>'
	        self.assertEqual( ht11 , ht1 , 'link doesn''t inserted correctly(or format is changed):\n '+ht1)
	        

        	#test type: html
	        fle2 = Dummy('just/test.html')
	        self.check_insert(t, fle2, ['OFS.DTMLDocument.DTMLDocument'], 'test.html')
	        #check pasteFile here
	        ht2= t.pasteFile('test.html', '<body></body>')
	        ht12='<body><a href="t/test.html">test</a></body>'
	        self.assertEqual( ht12 , ht2 , 'link doesn''t inserted correctly(or format is changed):\n '+ht2)
	        

        	#test type: dtml
	        fle3 = Dummy('just/test_html')
	        self.check_insert(t, fle3, ['OFS.DTMLDocument.DTMLDocument'], 'test_html')
	        #check pasteFile here
	        ht3= t.pasteFile('test_html', '<body></body>')
	        ht13='<body><a href="t/test_html">test_html</a></body>'
	        self.assertEqual( ht13 , ht3 , 'link doesn''t inserted correctly(or format is changed):\n '+ht3)
	        
        	
        	#test type: gif
	        fle4 = Dummy('just/test.gif')
	        self.check_insert(t, fle4, ['Products.Photo.Photo.Photo', 'OFS.Image.Image'], 'test.gif')
	        #check pasteFile here
	        ht4= t.pasteFile('test.gif', '<body></body>')
	        ht14='<body><img src="t/test.gif" border=0> </body>'
	        self.assertEqual( ht14 , ht4 , 'link doesn''t inserted correctly(or format is changed):\n '+ht4)

		#test objectIds, etc
		if not (('test.html' in t.objectIds('DTML Document') 
			and 'test_html' in t.objectIds('DTML Document')
			and len(t.objectIds('DTML Document')) == 2)):
				self.fail('Bad Object Id''s')
			
		self.assertEqual(len(t.objectValues('DTML Document')), 2, "Bad object values")
		
		self.assertEqual(len(t.objectItems('DTML Document')), 2, "Bad object items")
		

	        self.check_delete(t, fle1, 'test.txt', ht1)
	        self.check_delete(t, fle2, 'test.html', ht2)
	        self.check_delete(t, fle3, 'test_html', ht3)
	        self.check_delete(t, fle4, 'test.gif', ht4)

	
	def testManage_afterClone(self):
		#Not implemented yet
		pass
	
	
	def testRequestConfirmation(self):
		#Not implemented yet
		pass

	def testCancelConfirmationRequest(self):
		#Not implemented yet
		pass
	
	def testConfirm(self):
		#Not implemented yet
		pass
	
	
	def testRefuse(self):
		#Not implemented yet
		pass
	
	def testRating(self):
	        t = self.app.docs.storage.t
	        t.rate(4)

		self.assertEqual( t.getProperty('rate_sum', 0), 4 , 'Bad rate sum')
		self.assertEqual( t.getProperty('rate_users', []), ['Anonymous User'] , 'Bad rate user')
	        
	        self.assert_(t.rated(), 'rated() error')
	        self.assertEqual( t.get_rating(), 4 , ' get_rating error')

	        

	def testGetTasksFor(self):
		t = self.app.docs.storage.t
		f = t.getTasksFor(self.app.docs.storage)
		if not str(f.__class__).find('TaskContainer'):
			self.fail( 'Can not get TaskContainer' )

