#$Id: HeadingTests.py,v 1.12 2002/08/23 09:02:36 kfirsov Exp $

from ZopeTest import ZopeTestCase
import re


class ExampleTestCase(ZopeTestCase):
	head_id = ''
	
	def setUp(self):
	        from Products.CMFNauTools import Heading
       		import random
		ID = 'h'
		while getattr(self.app.docs.storage, ID, None):
			ID = 'test_heading_' + str( random.randrange(1000000) )

	        self.app.docs.storage.manage_addProduct['CMFNauTools'].\
	        	manage_addHeading(ID, title='test', REQUEST=None)
	        h = getattr(self.app.docs.storage, ID)
	        self.assert_(h, 'AddHeading Error' )
	        self.head_id = ID
                
	def tearDown(self):
		self.app.docs.storage.manage_delObjects(self.head_id)
		
	def test__call__(self):
		#Not implemented
		h = getattr(self.app.docs.storage, self.head_id)
		#print h() - does'n work: (No accessible views available 
		#				for /docs/storage/h)

	def testManage_afterClone(self):
		#Not implemented
		pass

	def testManage_permissions(self):
		from Products.CMFCore.utils import getToolByName
		
		h = getattr(self.app.docs.storage, self.head_id)
		
		# get all existent groups
		url_tool = getToolByName(h.content, 'portal_url')
		portal = url_tool.getPortalObject()
		groups = portal.acl_users.groups.keys()
		
		req = self.app.REQUEST
		# clear all permissions
		h.manage_permissions(req)
		self.assertEqual( h.get_local_group_roles(), (), \
			'Error while clearing permissions' )

		#set new permissions and test them
		req['Editor_0'] = groups[0]
		req['Author_0'] = groups[0]
		h.manage_permissions(req)
		self.assertNotEqual( h.get_local_group_roles(), (), \
			'Error after setting permissions' )
		self.assertEqual( h.get_local_group_roles()[0][0], groups[0], \
			'Invalid group name' )
		roles = list(h.get_local_group_roles()[0][1])
		roles.sort()
		self.assertEqual( roles, ['Author', 'Editor'], 'Invalid roles' )
		
		

	def testSetLocalRoles_GetLocalRoles_DelLocalRoles_UserRoles(self):
		from Products.CMFNauTools import HTMLDocument

		h = getattr(self.app.docs.storage, self.head_id)

		# first we create htmldocument into heading
		id = 'test_reindex_roles'
		h.manage_addProduct['CMFNauTools'].addHTMLDocument(id, 
				title='title', description='descr', text_format='html', 
				text='text goes here')
		t = getattr(h, id)
		
		#---test roles assignment---
		h.setLocalRoles('tester1', ('Writer', 'Reader'))
		h.setLocalRoles('tester2', ['Editor', 'Writer'])
		t1_h_roles = h.getLocalRoles('tester1')[0]
		t1_h_roles.sort()
		if  t1_h_roles != ['Reader', 'Writer']:
			self.fail('Bad roles for tester1 in Heading')
		t2_h_roles = h.getLocalRoles('tester2')[0]
		t2_h_roles.sort()
		if t2_h_roles != ['Editor', 'Writer']:
			self.fail('Bad roles for tester2 in Heading')

		t1_t_roles = t.getLocalRoles('tester1')[0]
		t1_t_roles.sort()
		if t1_t_roles != ['Reader', 'Writer']:
			self.fail('Bad roles for tester1 in subobject')
		t2_t_roles = t.getLocalRoles('tester2')[0]
		t2_t_roles.sort()
		if t2_t_roles != ['Editor', 'Writer']:
			self.fail('Bad roles for tester2 in subobject')
			
		h.delLocalRoles(['tester2'])
		
		self.assertEqual((), h.getLocalRoles('tester2'), \
			'Roles of tester2 was not deleted')
		self.assertNotEqual((), h.getLocalRoles('tester1'), \
			'Roles of tester1 was deleted')
		
		self.assertEqual((), t.getLocalRoles('tester2'), \
			'Roles of tester2 was not deleted in subobject')
		self.assertNotEqual((), t.getLocalRoles('tester1'), \
			'Roles of tester1 was deleted in subobject')
		
		h.setLocalRoles('Anonymous User', ['Editor', 'Author'])
		h_roles = h.user_roles()
		h_roles.sort()
		self.assertEqual( h_roles,  ['Author', 'Editor'], \
			'Bad roles for current user')
		
		t_roles = t.user_roles()
		t_roles.sort()
		self.assertEqual( t_roles,  ['Author', 'Editor'], \
			'Bad roles for current user in subobject')
		
		#test user_roles
		h.delLocalRoles(['Anonymous User'])
		self.assertEqual( len(h.user_roles()), 0, \
			'Current user roles removal error')
		self.assertEqual( len(t.user_roles()), 0, \
			'Current user roles removal error in subobject')

	def testReindexSubObjects(self):
		"""
		We test Reindex objects inside the given folder 
		(it updates 'allowedRolesAndUsers')
		"""

		def role_for_user(uname, roles_and_users):
			roles = {}
			cur_role = 'no roles'
			for role_or_user in roles_and_users:
				if role_or_user.find('user:') == -1:
					cur_role = role_or_user
				if role_or_user.find( 'user:'+uname ) != -1:
					roles[ cur_role ] = 1
			try:
				del roles['no roles']
			except:
				pass
			return roles


		from Products.CMFCore.utils import getToolByName
		from Products.CMFCore.PortalFolder import PortalFolder
		from Products.CMFNauTools import HTMLDocument
		
		h = getattr(self.app.docs.storage, self.head_id)
		
		# first we create htmldocument into heading
		id = 'test_reindex_roles'
		h.manage_addProduct['CMFNauTools'].addHTMLDocument(id, 
				title='title', description='descr', text_format='html', 
				text='text goes here')
		t = getattr(h, id)
		
		#delete all our roles and create role Reviewer to test reindexSubObjects()
		h.delLocalRoles(['Anonymous User'])

		h.manage_setLocalRoles('Anonymous User', ['Reviewer'])

		catalog = getToolByName(h, 'portal_catalog', None)


		#two possible valid ways: 1) can not find file 
		#   2)'user:Anonymous User' has no roles
		#
		#error may be only if : len (results)>0 + found exactly file 
		#   we've search + Anonymous User has any role

		results = catalog.searchResults(path= 'docs/storage/'+ \
										self.head_id, id=id)
		for result in results:
			#check roles and users
			roles_and_users = catalog.getIndexDataForRID(  \
								results[0].getRID() )['allowedRolesAndUsers']
			roles = role_for_user('Anonymous User', roles_and_users)
			self.failIf( len( roles.keys() ) > 0, \
				'Before reindexSubObjects() user has roles')

		h.reindexSubObjects()
		
		results = catalog.searchResults(path= 'docs/storage/'+ \
										self.head_id, id=id)
		for result in results:
			roles_and_users = catalog.getIndexDataForRID(  \
								results[0].getRID() )['allowedRolesAndUsers']
			roles = role_for_user('Anonymous User', roles_and_users)
			self.failIf('Reviewer' not in roles.keys(), \
				'After reindexSubObjects() user has no role Reviewer')
			self.assertEqual( len( roles.keys() ), 1, \
				'After reindexSubObjects() user has wrong roles')

		# delete role 'Rewiewer' and look what's happening:
		h.manage_delLocalRoles(['Anonymous User'])
		
		results = catalog.searchResults(path= 'docs/storage/'+ \
										self.head_id, id=id)
		for result in results:
			roles_and_users = catalog.getIndexDataForRID(  \
								results[0].getRID() )['allowedRolesAndUsers']
			roles = role_for_user('Anonymous User', roles_and_users)
			self.failIf('Reviewer' not in roles.keys(), \
				'Before reindexSubObjects() user has no role Reviewer')
			self.assertEqual( len( roles.keys() ), 1, \
				'Before reindexSubObjects() user has wrong roles')
		
		h.reindexSubObjects()

		results = catalog.searchResults(path= 'docs/storage/'+ \
										self.head_id, id=id)
		for result in results:
			roles_and_users = catalog.getIndexDataForRID(  \
								results[0].getRID() )['allowedRolesAndUsers']
			roles = role_for_user('Anonymous User', roles_and_users)
			self.failIf( len( roles.keys() ) > 0, \
				'After reindexSubObjects() roles was not removed')