#$Id: TrackerTests.py,v 1.5 2002/10/15 11:35:12 kfirsov Exp $

from ZopeTest import ZopeTestCase
import re


class TrackerTestCase(ZopeTestCase):
	tracker = None
	
	def setUp(self):
	        from Products.Tracker import Tracker
	        self.tracker = self.app.docs.tracker = Tracker.Tracker('tracker', 'tracker object')
                
	def tearDown(self):
		pass
		
	def test_add_del_regions(self):
		tracker_regions = self.tracker._regions
		self.tracker._regions={}
		self.assertEqual( len( self.tracker.getRegions().keys() ), 0 , 
			'Error startup test_add_del_regions() test' )
		name = 'TEST_NAME'
		start = '195.64.216.100'
		end   = '201.64.0.101'
		descr = 'This is test'
		self.tracker.add_regions(name=name, 
					begin=start, 
					end=end, 
					descr=descr, REQUEST=None)
		self.assertEqual( len( self.tracker.getRegions().keys() ), 1 , 
			'Error testing add_regions' )
		self.assert_( self.tracker.getRegions().has_key(name), 
			'Error testing add_regions'  )
		self.assertEqual( self.tracker.getRegions()[name], (start, end, descr), 
			'Error testing add_regions' )
		
		self.tracker.del_regions(region_key_del_data=[name], REQUEST=None)
		self.assertEqual( self.tracker.getRegions(), {}, 
			'Error testing del_regions' )
		self.tracker._regions = tracker_regions

	def test_del_resume_users_pages(self):
		test_users = ['user1', 'user2', 'user3', 'user4']
		self.tracker._page_user_data = {}
		self.tracker._page_all_data = {}
		for user in test_users:
			self.tracker._page_user_data[user+'_page'] = { user:{ test_users.index(user): test_users.index(user)+1} }
			self.tracker._page_all_data[user+'_page'] = { test_users.index(user): test_users.index(user)+1 }
			self.tracker._trackable_pages.append( user+'_page' )
		
		#test users
		for user in test_users:
			self.tracker.del_users( [user], completely=1)
			self.failIf( user in self.tracker._users, 'User not deleted')
			self.assert_( user in self.tracker._untrackable_users, 'Not user in user untrackable list' )
			self.failIf( self.tracker._page_user_data[ user+'_page'].has_key( user ), 
				'User not deleted from statistics')
			self.assertEqual( self.tracker._page_all_data[ user+'_page'][test_users.index(user)], 0,
				'Statistics was not updated')

		for user in test_users:
			self.tracker.resume_users([user])
			self.assert_( user not in self.tracker._untrackable_users, 'User not deleted from untrackable list' )

	def test_del_resume_pages(self):
		# test pages
		test_users = ['user1', 'user2', 'user3', 'user4']
		self.tracker._page_user_data = {}
		self.tracker._page_all_data = {}
		for user in test_users:
			self.tracker._page_user_data[user+'_page'] = { user:{ test_users.index(user): test_users.index(user)+1} }
			self.tracker._page_all_data[user+'_page'] = { test_users.index(user): test_users.index(user)+1 }
			self.tracker._trackable_pages.append( user+'_page' )
		
		for user in test_users:
			self.tracker.del_pages(pages_list=[ user+'_page' ], completely = 1)
			self.failIf( self.tracker._page_all_data.has_key( user+'_page' ),  'Page exists after removal')
			self.failIf( self.tracker._page_user_data.has_key( user+'_page' ), 'Page exists after removal')
			self.assert_( self.tracker._untrackable_pages.count( user+'_page' ), 'Page does not exist in untrackable')
		
		for user in test_users:
			self.tracker.resume_pages( [user+'_page'] )
			self.assert_( not self.tracker._untrackable_pages.count( user+'_page'),
				 'User not deleted from untrackable list' )


	def test_track_page(self):
		from DateTime import DateTime
		self.tracker._page_user_data = {}
		self.tracker._page_all_data = {}
		self.tracker._untrackable_pages=[]
		#self.tracker._regions = {}

		req = self.app.REQUEST
		req['AUTHENTICATED_USER'] = 'Anonymous User'
		req['REMOTE_ADDR'] = '195.64.216.109'
		date = DateTime()
		yy = date.year()
		mm = date.month()
		dd = date.day()
		hh = date.hour()
		self.tracker.track_page( 'test_url', req)
		
		self.assertEqual(  self.tracker.hitsForHour(yy, mm, dd, hh),  1, 'No History for hits')
		self.assertEqual( self.tracker.NumHostsForHour(yy, mm, dd, hh),  1, 'No History for hosts')
		self.assertEqual( self.tracker.groupsForHour(yy, mm, dd, hh),  1, 'No History for groups')
		

	def test_reset(self):
		self.tracker._page_user_data['page'] = { 'user':{ 1: 1} }
		self.tracker._page_all_data['page'] = { 1: 1 }
		self.tracker._trackable_pages=('trackable_page')
		self.tracker._untrackable_pages=('untrackable_page')
		self.tracker.reset()
		self.assertEqual(self.tracker._page_user_data, {}, '_page_user_data is not empty')
		self.assertEqual(self.tracker._page_all_data, {}, '_page_all_data is not empty')
		self.assertEqual(self.tracker._trackable_pages, [], '_trackable_pages is not empty')
		self.assertEqual(self.tracker._untrackable_pages, [], '_untrackable_pages is not empty')

	def test_turn_on_off(self):
		self.tracker.turn_on_off('on')
		self.assertEqual(self.tracker._active, 1, 'error')
		self.tracker.turn_on_off('on')
		self.assertEqual(self.tracker._active, 1, 'error')
		self.tracker.turn_on_off('off')
		self.assertEqual(self.tracker._active, 0, 'error')
		self.tracker.turn_on_off('off')
		self.assertEqual(self.tracker._active, 0, 'error')
		self.tracker.turn_on_off('on')
		self.assertEqual(self.tracker._active, 1, 'error')
		
	def test_hostsForPeriod(self):
		self.tracker._hosts_count = {}
		self.tracker._hosts_count[1] =2
		self.tracker._hosts_count[2] =3
		self.tracker._hosts_count[4] =1
		self.tracker._hosts_count[5] =4
		self.assertEqual( self.tracker.hostsForPeriod(1, 2), 2, 'Error testing hostsForPeriod')
		self.assertEqual( self.tracker.hostsForPeriod(1, 3), 5, 'Error testing hostsForPeriod')
		self.assertEqual( self.tracker.hostsForPeriod(1, 5), 6, 'Error testing hostsForPeriod')
		self.assertEqual( self.tracker.hostsForPeriod(1, 6), 10, 'Error testing hostsForPeriod')
		self.assertEqual( self.tracker.hostsForPeriod(3, 5), 1, 'Error testing hostsForPeriod')
		self.assertEqual( self.tracker.hostsForPeriod(3, 6), 5, 'Error testing hostsForPeriod')
	
	def test_groupsForPeriod_HitsForPeriod(self):
		self.tracker._page_all_data = {'page1': {1:2}, 'page2': {10:3}, 'page3': {15:1}, 'page4': {12:5}}
		self.tracker._page_user_data = {'page1' : {'user1' : {1:2} }, 
			'page2' : {'user2' : {10:3} },
			'page3' : {'user1' : {15:1} },
			'page4' : {'user2' : {12:5} },
			 }
		self.tracker._trackable_pages = ['page1', 'page2', 'page3', 'page4']
		self.tracker._users = ['user1', 'user2']
 		
		self.assertEqual(self.tracker.groupsForPeriod(0, 1), 0, 'error in groupsForPeriod')
		self.assertEqual(self.tracker.groupsForPeriod(0, 7), 1, 'error in groupsForPeriod')
		self.assertEqual(self.tracker.groupsForPeriod(1, 12), 2, 'error in groupsForPeriod')
		self.assertEqual(self.tracker.groupsForPeriod(1, 16), 2, 'error in groupsForPeriod')
		self.assertEqual(self.tracker.groupsForPeriod(7, 13), 1, 'error in groupsForPeriod')
		self.assertEqual(self.tracker.groupsForPeriod(15, 100), 1, 'error in groupsForPeriod')
		self.assertEqual(self.tracker.groupsForPeriod(16, 100), 0, 'error in groupsForPeriod')
		
		self.assertEqual(self.tracker.HitsForPeriod(0, 1), 0, 'error in HitsForPeriod')
		self.assertEqual(self.tracker.HitsForPeriod(0, 7), 2, 'error in HitsForPeriod')
		self.assertEqual(self.tracker.HitsForPeriod(1, 12), 5, 'error in HitsForPeriod')
		self.assertEqual(self.tracker.HitsForPeriod(1, 16), 11, 'error in HitsForPeriod')
		self.assertEqual(self.tracker.HitsForPeriod(7, 13), 8, 'error in HitsForPeriod')
		self.assertEqual(self.tracker.HitsForPeriod(15, 100), 1, 'error in HitsForPeriod')
		self.assertEqual(self.tracker.HitsForPeriod(16, 100), 0, 'error in HitsForPeriod')

	def test_Statistics(self):
		from DateTime import DateTime
		h1 = 2000*10000+1*24
		h2 = 2000*10000+10*24
		h3 = 2000*10000+15*24
		h4 = 2000*10000+12*24
		self.tracker._page_all_data = {'page1': {h1:2}, 'page2': {h2:3, h4:5}, 'page3': {h3:1}}
		self.tracker._page_user_data = {'page1' : {'user1' : {h1:2} }, 
			'page2' : {'user2' : {h2:3} },
			'page3' : {'user1' : {h3:1} },
			'page2' : {'user2' : {h4:5} },
			 }
		self.tracker._trackable_pages = ['page1', 'page2', 'page3']
		self.tracker._users = ['user1', 'user2']
		
		time1 = DateTime(2000, 1, 1)
		time2 = DateTime(2000, 1, 30)
		stime1 = str(time1.year())+'/'+str(time1.month())+'/'+str(time1.day())
		stime2 = str(time2.year())+'/'+str(time2.month())+'/'+str(time2.day())
		
		#print self.tracker.Statistics('page2', None, stime1, stime2)
		self.assertEqual(self.tracker.Statistics('page2', None, stime1, stime2), 8, 'error in Statistics')

		time1 = DateTime(2000, 1, 12)
		stime1 = str(time1.year())+'/'+str(time1.month())+'/'+str(time1.day())
		self.assertEqual(self.tracker.Statistics('page2', None, stime1, None), 5, 'error in Statistics')
		self.assertEqual(self.tracker.Statistics('page2', None, None, stime1), 3, 'error in Statistics')
		self.assertEqual(self.tracker.Statistics('page9', None, None, None), 0, 'error in Statistics')
		
		self.assertEqual(self.tracker.Statistics('page3', 'user89', stime1, stime2), 0, 'error in Statistics')
		self.assertEqual(self.tracker.Statistics('page3', 'user1', stime1, stime2), 1, 'error in Statistics')

	def test_NumHostsForHour(self):
		self.tracker._hosts_count = {}
		self.tracker._hosts_count[20020000 + 1*24 + 1] =2
		self.tracker._hosts_count[20020000 + 1*24 + 2] =3
		self.tracker._hosts_count[20020000 + 1*24 + 4] =1
		self.tracker._hosts_count[20020000 + 1*24 + 5] =4

		self.assertEqual( self.tracker.NumHostsForHour(2002, 1, 1, 1), 2 ,
			'Error testing NumHostsForHour')
		self.assertEqual( self.tracker.NumHostsForHour(2002, 1, 1, 2), 3 ,
			'Error testing NumHostsForHour')
		self.assertEqual( self.tracker.NumHostsForHour(2002, 1, 1, 3), 0 ,
			'Error testing NumHostsForHour')
		self.assertEqual( self.tracker.NumHostsForHour(2002, 1, 1, 4), 1 ,
			'Error testing NumHostsForHour')


	def test_groupsForHour_hitsForHour(self):
		from DateTime import DateTime
		h1 = 2000*10000+1*24
		h2 = 2000*10000+10*24
		h3 = 2000*10000+15*24
		h4 = 2000*10000+12*24
		self.tracker._page_all_data = {'page1': {h1:2}, 'page2': {h2:3, h4:5, h1:100}, 'page3': {h3:1}}
		self.tracker._page_user_data = {'page1' : {'user1' : {h1:2} }, 
			'page2' : {'user2' : {h2:3, h4:5, h1:100} },
			'page3' : {'user1' : {h3:1} },
			 }
		self.tracker._trackable_pages = ['page1', 'page2', 'page3']
		self.tracker._users = ['user1', 'user2']

		self.assertEqual(self.tracker.groupsForHour(2000, 1, 1, 0, None, None), 2, 'Error in groupsForHour')
		self.assertEqual(self.tracker.groupsForHour(2000, 1, 1, 1, None, None), 0, 'Error in groupsForHour')
		self.assertEqual(self.tracker.groupsForHour(2000, 1, 1, 0, ['page1', 'page2'], None), 2, 'Error in groupsForHour')
		self.assertEqual(self.tracker.groupsForHour(2000, 1, 1, 0, ['page1', 'page2'], ['user6']), 0, 'Error in groupsForHour')
		self.assertEqual(self.tracker.groupsForHour(2000, 1, 1, 0, ['page10'], ['user1']), 0, 'Error in groupsForHour')
		self.assertEqual(self.tracker.groupsForHour(2000, 1, 1, 0, ['page1', 'page2'], ['user1']), 1, 'Error in groupsForHour')

		self.assertEqual(self.tracker.hitsForHour(2000, 1, 1, 0, None, None), 102, 'Error in hitsForHour')
		self.assertEqual(self.tracker.hitsForHour(2000, 1, 1, 1, None, None), 0, 'Error in hitsForHour')
		self.assertEqual(self.tracker.hitsForHour(2000, 1, 1, 0, ['page1', 'page2'], None), 102, 'Error in hitsForHour')
		self.assertEqual(self.tracker.hitsForHour(2000, 1, 1, 0, ['page1', 'page2'], ['user6']), 0, 'Error in hitsForHour')
		self.assertEqual(self.tracker.hitsForHour(2000, 1, 1, 0, ['page10'], ['user1']), 0, 'Error in hitsForHour')
		self.assertEqual(self.tracker.hitsForHour(2000, 1, 1, 0, ['page1', 'page2'], ['user1']), 2, 'Error in hitsForHour')
		
	def test_NumHostsForToday(self):
		from DateTime import DateTime
		now = DateTime()
		hour = now.year() *10000 + now.dayOfYear()*24
		self.tracker._last_day_hosts = {}
		self.tracker._last_day_hosts = {hour : [192068200103L, 192068200104L, 192068200105L]}
		self.tracker._last_day_hosts[hour+1]=[192068200103L, 192068200106L, 192068200107L]
		self.assertEqual( self.tracker.NumHostsForToday(), 5, 'Error testing NumHostsForToday' )

	def test_NumHostsForYesterday(self):
		from DateTime import DateTime
		yesterday = DateTime()-1
		hour = yesterday.year() *10000 + yesterday.dayOfYear()*24
		self.tracker._last_day_hosts = {}
		self.tracker._last_day_hosts = {hour : [192068200103L, 192068200104L, 192068200105L]}
		self.tracker._last_day_hosts[hour+1]=[192068200103L, 192068200105L, 192068200107L]
		self.assertEqual( self.tracker.NumHostsForYesterday(), 4, 'Error testing NumHostsForToday' )

	def test_HostsSummary(self):
		from DateTime import DateTime
		now = DateTime()
		hour = now.year() *10000 + now.dayOfYear()*24
		self.tracker._last_day_hosts = {}
		self.tracker._last_day_hosts = {hour : [192068200103L, 192068200104L, 192068200105L]}
		self.tracker._last_day_hosts[hour+1]=[192068200103L, 192068200106L, 192068200107L]
		
		yesterday = now -1
		hour = yesterday.year() *10000 + yesterday.dayOfYear()*24
		self.tracker._last_day_hosts[hour] = [192068200103L, 192068200104L, 192068200105L]
		self.tracker._last_day_hosts[hour+1]=[192068200103L, 192068200105L, 192068200107L]
		
		day3ago = now-3
		day10ago = now-10
		self.tracker._hosts_count = {}
		hour = day3ago.year()*10000 + day3ago.dayOfYear()*24
		self.tracker._hosts_count[hour ] =2
		self.tracker._hosts_count[hour +1] =3
		self.tracker._hosts_count[hour +10] =1

		hour = day10ago.year()*10000 + day10ago.dayOfYear()*24
		self.tracker._hosts_count[hour ] =3
		self.tracker._hosts_count[hour +1] =5
		self.tracker._hosts_count[hour +10] =2
		
		self.assertEqual( self.tracker.HostsSummary(), (5, 4, 6, 16), 'Error testing HostsSummary' )



	def test_UsersSummary_hitsSummary(self):
		from DateTime import DateTime
		today = DateTime()
		yesterday = today - 1
		days_5_ago = today - 5
		days_20_ago = today - 20
		h1 = int (today.year()*10000 + today.dayOfYear()*24 + today.hour())
		h2 = int (yesterday.year()*10000 + yesterday.dayOfYear()*24 + yesterday.hour())
		h3 = int (days_5_ago.year()*10000 + days_5_ago.dayOfYear()*24 + days_5_ago.hour())
		h4 = int (days_20_ago.year()*10000 + days_20_ago.dayOfYear()*24 + days_20_ago.hour())
		self.tracker._trackable_pages = ['page1', 'page2', 'page3']
		self.tracker._users = ['user1', 'user2']
		self.tracker._page_all_data = {'page1': {h1:2}, 'page2': {h2:3, h4:5, h1:100}, 'page3': {h3:1}}
		self.tracker._page_user_data = {'page1' : {'user1' : {h1:2} }, 
			'page2' : {'user2' : {h2:3, h4:5, h1:100} },
			'page3' : {'user1' : {h3:1} },
			 }
		self.assertEqual( self.tracker.UsersSummary( pages=None, users=None), 
				(2, 1, 2, 2), 
				'Error in UsersSummary')
		self.assertEqual( self.tracker.UsersSummary( pages=['page1'], users=None), 
				(1, 0, 1, 1), 
				'Error in UsersSummary')
		self.assertEqual( self.tracker.UsersSummary( pages=['page1', 'page2'], users=['user1']), 
				(1, 0, 1, 1), 
				'Error in UsersSummary')
		self.assertEqual( self.tracker.UsersSummary( pages=['page1', 'page2'], users=['user2']), 
				(1, 1, 1, 1), 
				'Error in UsersSummary')
		self.assertEqual( self.tracker.UsersSummary( pages=['page1', 'page2'], users=['user1', 'user2']), 
				(2, 1, 2, 2), 
				'Error in UsersSummary')

		self.assertEqual( self.tracker.hitsSummary( pages=None, users=None), 
				(102, 3, 106, 111), 
				'Error in hitsSummary')
		self.assertEqual( self.tracker.hitsSummary( pages=['page1'], users=None), 
				(2, 0, 2, 2), 
				'Error in hitsSummary')
		self.assertEqual( self.tracker.hitsSummary( pages=['page1', 'page2'], users=['user1']), 
				(2, 0, 2, 2), 
				'Error in hitsSummary')
		self.assertEqual( self.tracker.hitsSummary( pages=['page1', 'page2'], users=['user2']), 
				(100, 3, 103, 108), 
				'Error in hitsSummary')
		self.assertEqual( self.tracker.hitsSummary( pages=['page1', 'page2'], users=['user1', 'user2']), 
				(102, 3, 105, 110), 
				'Error in hitsSummary')


	def test_hostsForDay(self):
		self.tracker._hosts_count = {}
		self.tracker._hosts_count[20020000 + 1*24 + 1] =2
		self.tracker._hosts_count[20020000 + 1*24 + 2] =3
		self.tracker._hosts_count[20020000 + 1*24 + 4] =1
		self.tracker._hosts_count[20020000 + 2*24 + 5] =4
		self.assertEqual( self.tracker.hostsForDay(2002, 1, 1), 6, 
			'Error testing hostsForDay')
		self.assertEqual( self.tracker.hostsForDay(2002, 1, 2), 4, 
			'Error testing hostsForDay')
	
	def test_groupsForDay_hitsForDay(self):
		from DateTime import DateTime
		h1 = 2000*10000+1*24
		h2 = 2000*10000+10*24
		h3 = 2000*10000+15*24
		h4 = 2000*10000+12*24
		self.tracker._page_all_data = {'page1': {h1:2}, 'page2': {h2:3, h4:5, h1:100}, 'page3': {h3:1}}
		self.tracker._page_user_data = {'page1' : {'user1' : {h1:2} }, 
			'page2' : {'user2' : {h2:3, h4:5, h1:100} },
			'page3' : {'user1' : {h3:1} },
			 }
		self.tracker._trackable_pages = ['page1', 'page2', 'page3']
		self.tracker._users = ['user1', 'user2']

		self.assertEqual( self.tracker.groupsForDay(2000, 1, 1, None, None), 2, 
				'Error in groupsForDay')
		self.assertEqual( self.tracker.groupsForDay(2000, 1, 1, ['page1', 'page2'], None), 2, 
				'Error in groupsForDay')
		self.assertEqual( self.tracker.groupsForDay(2000, 1, 1, ['page1', 'page2'], ['user6']), 0, 
				'Error in groupsForDay')
		self.assertEqual( self.tracker.groupsForDay(2000, 1, 1, ['page10'], ['user1']), 0, 
				'Error in groupsForDay')
		self.assertEqual( self.tracker.groupsForDay(2000, 1, 1, ['page1', 'page2'], ['user1']), 1, 
				'Error in groupsForDay')

		self.assertEqual( self.tracker.hitsForDay(2000, 1, 1, None, None), 102, 
				'Error in hitsForDay')
		self.assertEqual( self.tracker.hitsForDay(2000, 1, 1, ['page1', 'page2'], None), 102, 
				'Error in hitsForDay')
		self.assertEqual( self.tracker.hitsForDay(2000, 1, 1, ['page1', 'page2'], ['user6']), 0, 
				'Error in hitsForDay')
		self.assertEqual( self.tracker.hitsForDay(2000, 1, 1, ['page10'], ['user1']), 0, 
				'Error in hitsForDay')
		self.assertEqual( self.tracker.hitsForDay(2000, 1, 1, ['page1', 'page2'], ['user1']), 2, 
				'Error in hitsForDay')

	def test_hostsForMonth(self):
		self.tracker._hosts_count = {}
		self.tracker._hosts_count[20020000 + 1*24 + 1] =2
		self.tracker._hosts_count[20020000 + 2*24 + 2] =3
		self.tracker._hosts_count[20020000 + 19*24 + 4] =1
		self.tracker._hosts_count[20020000 + 33*24 + 5] =4
		self.tracker._hosts_count[20020000 + 36*24 + 1] =4

		self.assertEqual( self.tracker.hostsForMonth(2002, 1), 6, 
			'Error testing hostsForMonth')
		self.assertEqual( self.tracker.hostsForMonth(2002, 2), 8, 
			'Error testing hostsForMonth')


	def test_groupsForMonth_hitsForMonth(self):
		from DateTime import DateTime
		h1 = 2000*10000+1*24 
		h2 = 2000*10000+40*24 # feb 2000
		h3 = 2000*10000+15*24
		h4 = 2000*10000+12*24
		self.tracker._page_all_data = {'page1': {h1:2}, 'page2': {h2:3, h4:5, h1:100}, 'page3': {h3:1}}
		self.tracker._page_user_data = {'page1' : {'user1' : {h1:2} }, 
			'page2' : {'user2' : {h2:3, h4:5, h1:100} },
			'page3' : {'user1' : {h3:1} },
			 }
		self.tracker._trackable_pages = ['page1', 'page2', 'page3']
		self.tracker._users = ['user1', 'user2']

		self.assertEqual( self.tracker.groupsForMonth(2000, 1, None, None), 2, 
				'Error in groupsForMonth)')
		self.assertEqual( self.tracker.groupsForMonth(2000, 2, None, None), 1, 
				'Error in groupsForMonth')
		self.assertEqual( self.tracker.groupsForMonth(2000, 1, ['page1', 'page2'], None), 2, 
				'Error in groupsForMonth')
		self.assertEqual( self.tracker.groupsForMonth(2000, 1, ['page1', 'page2'], ['user6']), 0, 
				'Error in groupsForMonth')
		self.assertEqual( self.tracker.groupsForMonth(2000, 1, ['page10'], ['user1']), 0, 
				'Error in groupsForMonth')
		self.assertEqual( self.tracker.groupsForMonth(2000, 1, ['page1', 'page2'], ['user1']), 1, 
				'Error in groupsForMonth')

		self.assertEqual( self.tracker.hitsForMonth(2000, 1, None, None), 108, 
				'Error in hitsForMonth')
		self.assertEqual( self.tracker.hitsForMonth(2000, 2, None, None), 3, 
				'Error in hitsForMonth')
		self.assertEqual( self.tracker.hitsForMonth(2000, 1, ['page1', 'page2'], None), 107, 
				'Error in hitsForMonth')
		self.assertEqual( self.tracker.hitsForMonth(2000, 1, ['page1', 'page2'], ['user6']), 0, 
				'Error in hitsForMonth')
		self.assertEqual( self.tracker.hitsForMonth(2000, 1, ['page10'], ['user1']), 0, 
				'Error in hitsForMonth')
		self.assertEqual( self.tracker.hitsForMonth(2000, 1, ['page1', 'page2'], ['user1']), 2, 
				'Error in hitsForMonth')

#################################################

	def test_cutPortalUrl(self):
		item = '/docs/external/go/tracker'
		#self.assertEqual( self.tracker.cutPortalUrl( item ), '????', 'cutPortalUrl works wrong')
		
	def test_getParentURL(self):
		item = '/docs/external/go/tracker'
		#self.assertEqual( self.tracker.getParentURL(item), '/docs/external' , 'getParentURL works wrong')
		
	def test_getID(self):
		item = '/docs/external/go/tracker'
		#self.assertEqual( self.tracker.getID(item), 'tracker', 'getID works wrong')
	
	




